@@ -74,7 +74,7 @@ int mapcolor_frnd;    // friendly sprite color
 int mapcolor_enemy;   // enemy sprite color
 int mapcolor_hair;    // crosshair color
 int mapcolor_sngl;    // single player arrow color
-int mapcolor_plyr[4] = { 112, 88, 64, 176 }; // colors for player arrows in multiplayer
+int mapcolor_plyr[4] = { 112, 88, 64, 32 }; // colors for player arrows in multiplayer
 
 //jff 3/9/98 add option to not show secret sectors until entered
 int map_secret_after;
@@ -1380,13 +1380,13 @@ static void AM_drawPlayers(void)
   for (i=0;i<MAXPLAYERS;i++) {
     player_t* p = &players[i];
 
-    if ( (deathmatch && !singledemo) && p != plr)
+    if ( (deathmatch && !demoplayback) && p != plr)
       continue;
 
     if (playeringame[i]) {
       fixed_t x = p->mo->x >> FRACTOMAPBITS, y = p->mo->y >> FRACTOMAPBITS;//e6y
       if (automapmode & am_rotate)
-  AM_rotate(&x, &y, ANG90-plr->mo->angle, plr->mo->x >> FRACTOMAPBITS, plr->mo->y >> FRACTOMAPBITS);
+        AM_rotate(&x, &y, ANG90-plr->mo->angle, plr->mo->x, plr->mo->y);
 
       AM_drawLineCharacter (player_arrow, NUMPLYRLINES, 0, p->mo->angle,
           p->powers[pw_invisibility] ? 246 /* *close* to black */
