@@ -36,6 +36,21 @@
  *-----------------------------------------------------------------------------
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+#include <sys/types.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+#ifdef HAVE_SYS_WAIT_H
+#include <sys/wait.h>
+#endif
+
+#ifdef USE_SDL_NET
+ #include "SDL.h"
+#endif
+
 #include "doomtype.h"
 #include "doomstat.h"
 #include "d_net.h"
@@ -55,17 +70,6 @@
 #include "r_fps.h"
 #include "lprintf.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-#include <sys/types.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
-#endif
-
 static boolean   server;
 static int       remotetic; // Tic expected from the remote
 static int       remotesend; // Tic expected by the remote
@@ -167,6 +171,7 @@ void D_InitNetGame (void)
   doomcom->consoleplayer = 0;
   doomcom->numnodes = 0; doomcom->numplayers = 1;
   localcmds = netcmds[consoleplayer];
+  netgame = (M_CheckParm("-solo-net") != 0);
 
   for (i=0; i<doomcom->numplayers; i++)
     playeringame[i] = true;
@@ -364,7 +369,7 @@ void NetUpdate(void)
 }
 #else
 
-static void D_BuildNewTiccmds()
+void D_BuildNewTiccmds(void)
 {
     static int lastmadetic;
     int newtics = I_GetTime() - lastmadetic;
@@ -464,12 +469,15 @@ void TryRunTics (void)
     runtics = (server ? remotetic : maketic) - gametic;
     if (!runtics) {
       if (!movement_smooth) {
+#ifdef HAVE_NET
         if (server)
           I_WaitForPacket(ms_to_next_tick);
         else
+#endif
           I_uSleep(ms_to_next_tick*1000);
       }
       if (I_GetTime() - entertime > 10) {
+#ifdef HAVE_NET
         if (server) {
           char buf[sizeof(packet_header_t)+1];
           remotesend--;
@@ -477,6 +485,7 @@ void TryRunTics (void)
           buf[sizeof(buf)-1] = consoleplayer;
           I_SendPacket((packet_header_t *)buf, sizeof buf);
         }
+#endif
         M_Ticker(); return;
       }
       //if ((displaytime) < (tic_vars.next-SDL_GetTicks()))
