@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2006 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,31 +23,33 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
- * Dehacked file support
- * New for the TeamTNT "Boom" engine
+ * DESCRIPTION:
+ *  Dehacked file support
+ *  New for the TeamTNT "Boom" engine
+ * 
+ *  Author: Ty Halderman, TeamTNT
  *
- * Author: Ty Halderman, TeamTNT
+ *  Description: This file translates the #defined string constants
+ *  to named variables to externalize them for deh/bex changes.
+ *  Should be able to compile with D_FRENCH (for example) and still
+ *  work (untested).
  *
- * Description: This file translates the #defined string constants
- * to named variables to externalize them for deh/bex changes.
- * Should be able to compile with D_FRENCH (for example) and still
- * work (untested).
- *
- *-----------------------------------------------------------------------------
-*/
+ */
 
 #ifndef __D_DEH__
 #define __D_DEH__
 
+void ProcessDehFile(const char *filename, const char *outfilename, int lumpnum);
+
 //
 //      Ty 03/22/98 - note that we are keeping the english versions and
 //      comments in this file
 //      New string names all start with an extra s_ to avoid conflicts,
 //      but are otherwise identical to the original including uppercase.
-//      This is partly to keep the changes simple and partly for easier 
+//      This is partly to keep the changes simple and partly for easier
 //      identification of the locations in which they're used.
 //
 //      Printed strings for translation
@@ -590,16 +594,6 @@ extern const char *s_HUSTR_PLRBROWN; // = HUSTR_PLRBROWN;
 //#define HUSTR_PLRRED          "Red: "
 extern const char *s_HUSTR_PLRRED; // = HUSTR_PLRRED;
 
-// Ty - Note these are chars, not char *, so name is sc_XXX
-//#define HUSTR_KEYGREEN        'g'
-extern char sc_HUSTR_KEYGREEN; // = HUSTR_KEYGREEN;
-//#define HUSTR_KEYINDIGO       'i'
-extern char sc_HUSTR_KEYINDIGO; // = HUSTR_KEYINDIGO;
-//#define HUSTR_KEYBROWN        'b'
-extern char sc_HUSTR_KEYBROWN; // = HUSTR_KEYBROWN;
-//#define HUSTR_KEYRED  'r'
-extern char sc_HUSTR_KEYRED; // = HUSTR_KEYRED;
-
 //
 //      AM_map.C
 //
@@ -658,11 +652,6 @@ extern const char* s_STSTR_CHOPPERS; // = STSTR_CHOPPERS;
 //#define STSTR_CLEV            "Changing Level..."
 extern const char* s_STSTR_CLEV; // = STSTR_CLEV;
 
-//#define STSTR_COMPON    "Compatibility Mode On"            // phares
-extern const char* s_STSTR_COMPON; // = STSTR_COMPON;
-//#define STSTR_COMPOFF   "Compatibility Mode Off"           // phares
-extern const char* s_STSTR_COMPOFF; // = STSTR_COMPOFF;
-
 //
 //      F_Finale.C
 //
@@ -864,7 +853,7 @@ extern const char* s_C5TEXT; // = C5TEXT;
 extern const char* s_C6TEXT; // = C6TEXT;
 
 
-// after map 06 
+// after map 06
 
 /*
 #define P1TEXT  \
@@ -1124,8 +1113,6 @@ extern const char* startup5;
 // from g_game.c, prefix for savegame name like "boomsav"
 extern const char* savegamename;
 
-extern boolean deh_loaded; // sf
-
 void D_BuildBEXTables(void);
 
 #endif
