@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,12 +23,12 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
  *  Event information structures.
- *    
+ *
  *-----------------------------------------------------------------------------*/
 
 
@@ -47,19 +49,19 @@ typedef enum
   ev_keydown,
   ev_keyup,
   ev_mouse,
-  ev_axis
+  ev_joystick
 } evtype_t;
 
 // Event structure.
 typedef struct
 {
   evtype_t  type;
-  int       data1;    // keys / mouse/joystick buttons; device number
-  int       data2;    // mouse x move; axis number
-  int       data3;    // mouse y move; axis value
+  int       data1;    // keys / mouse/joystick buttons
+  int       data2;    // mouse/joystick x move
+  int       data3;    // mouse/joystick y move
 } event_t;
 
- 
+
 typedef enum
 {
   ga_nothing,
@@ -71,8 +73,6 @@ typedef enum
   ga_completed,
   ga_victory,
   ga_worlddone,
-  ga_screenshot,
-  ga_loadhublevel,   // sf: for hubs
 } gameaction_t;
 
 
@@ -91,7 +91,7 @@ typedef enum
   // Flag: game events, not really buttons.
   BT_SPECIAL      = 128,
   BT_SPECIALMASK  = 3,
-    
+
   // Flag, weapon change pending.
   // If true, the next 4 bits hold weapon num.
   BT_CHANGE       = 4,
@@ -109,21 +109,16 @@ typedef enum
   BTS_SAVEGAME    = 2,
   BTS_RESTARTLEVEL= 3, // Restarts the current level
 
-  // Savegame slot numbers occupy the second byte of buttons.    
+  // Savegame slot numbers occupy the second byte of buttons.
   BTS_SAVEMASK    = (4+8+16),
   BTS_SAVESHIFT   = 2,
-  
+
 } buttoncode_t;
 
 
 //
 // GLOBAL VARIABLES
 //
-#define MAXEVENTS               64
-
-extern event_t   events[MAXEVENTS];
-extern int       eventhead;
-extern int       eventtail;
 
 extern gameaction_t gameaction;
 
