@@ -207,6 +207,7 @@ extern  int leveltime;  // tics in game play for par
 extern  boolean usergame;
 extern  boolean demoplayback;
 extern  boolean demorecording;
+extern  int demover;
 
 // Quit after playing a demo from cmdline.
 extern  boolean   singledemo;
@@ -297,8 +298,10 @@ extern int default_weapon_recoil;
 extern int player_bobbing;  // whether player bobs or not   // phares 2/25/98
 extern int default_player_bobbing;  // killough 3/1/98: make local to each game
 
+#ifdef DOGS
 extern int dogs, default_dogs;     // killough 7/19/98: Marine's best friend :)
 extern int dog_jumping, default_dog_jumping;   // killough 10/98
+#endif
 
 /* killough 8/8/98: distance friendly monsters tend to stay from player */
 extern int distfriend, default_distfriend;
