@@ -93,6 +93,7 @@
 static size_t   savegamesize = SAVEGAMESIZE; // killough
 static boolean  netdemo;
 static const byte *demobuffer;   /* cph - only used for playback */
+static int demolength; // check for overrun (missing DEMOMARKER)
 static FILE    *demofp; /* cph - record straight to file */
 static const byte *demo_p;
 static short    consistancy[MAXPLAYERS][BACKUPTICS];
@@ -125,6 +126,7 @@ int             basetic;       /* killough 9/29/98: for demo sync */
 int             totalkills, totallive, totalitems, totalsecret;    // for intermission
 boolean         demorecording;
 boolean         demoplayback;
+int             demover;
 boolean         singledemo;           // quit after playing a demo from cmdline
 wbstartstruct_t wminfo;               // parms for world map / intermission
 boolean         haswolflevels = false;// jff 4/18/98 wolf levels present
@@ -257,7 +259,7 @@ int    bodyqueslot, bodyquesize;        // killough 2/8/98
 mobj_t **bodyque = 0;                   // phares 8/10/98
 
 static void G_DoSaveGame (boolean menu);
-static const byte* G_ReadDemoHeader(const byte* demo_p);
+static const byte* G_ReadDemoHeader(const byte* demo_p, size_t size, boolean failonerror);
 
 //
 // G_BuildTiccmd
@@ -613,7 +615,8 @@ static void G_DoLoadLevel (void)
   }
 
   P_SetupLevel (gameepisode, gamemap, 0, gameskill);
-  displayplayer = consoleplayer;    // view the guy you are playing
+  if (!demoplayback) // Don't switch views if playing a demo
+    displayplayer = consoleplayer;    // view the guy you are playing
   gameaction = ga_nothing;
   Z_CheckHeap ();
 
@@ -628,19 +631,6 @@ static void G_DoLoadLevel (void)
   // killough 5/13/98: in case netdemo has consoleplayer other than green
   ST_Start();
   HU_Start();
-
-  // killough: make -timedemo work on multilevel demos
-  // Move to end of function to minimize noise -- killough 2/22/98:
-
-  if (timingdemo)
-    {
-      static int first=1;
-      if (first)
-        {
-          starttime = I_GetTime_RealTime ();
-          first=0;
-        }
-    }
 }
 
 
@@ -932,6 +922,12 @@ void G_Ticker (void)
     prevgamestate = gamestate;
   }
 
+  // e6y
+  // do nothing if a pause has been pressed during playback
+  // pausing during intermission can cause desynchs without that
+  if (paused & 2 && gamestate != GS_LEVEL)
+    return;
+
   // do main actions
   switch (gamestate)
     {
@@ -1375,6 +1371,15 @@ void G_DoCompleted (void)
   gamestate = GS_INTERMISSION;
   automapmode &= ~am_active;
 
+  // lmpwatch.pl engine-side demo testing support
+  // print "FINISHED: <mapname>" when the player exits the current map
+  if (nodrawers && (demoplayback || timingdemo)) {
+    if (gamemode == commercial)
+      lprintf(LO_INFO, "FINISHED: MAP%02d\n", gamemap);
+    else
+      lprintf(LO_INFO, "FINISHED: E%dM%d\n", gameepisode, gamemap);
+  }
+
   WI_Start (&wminfo);
 }
 
@@ -1937,7 +1942,7 @@ void G_Compatibility(void)
     // comp_pursuit - MBF AI change, limited pursuit?
     { mbf_compatibility, mbf_compatibility },
     // comp_doorstuck - monsters stuck in doors fix
-    { boom_compatibility, mbf_compatibility },
+    { boom_202_compatibility, mbf_compatibility },
     // comp_staylift - MBF AI change, monsters try to stay on lifts
     { mbf_compatibility, mbf_compatibility },
     // comp_zombie - prevent dead players triggering stuff
@@ -1977,13 +1982,16 @@ void G_Compatibility(void)
     monster_friction = 0;
     help_friends = 0;
 
+#ifdef DOGS
     dogs = 0;
     dog_jumping = 0;
+#endif
 
     monkeys = 0;
   }
 }
 
+#ifdef DOGS
 /* killough 7/19/98: Marine's best friend :) */
 static int G_GetHelpers(void)
 {
@@ -1993,6 +2001,7 @@ static int G_GetHelpers(void)
     j = M_CheckParm ("-dogs");
   return j ? j+1 < myargc ? atoi(myargv[j+1]) : 1 : default_dogs;
 }
+#endif
 
 // killough 3/1/98: function to reload all the default parameter
 // settings before a new game begins
@@ -2016,8 +2025,10 @@ void G_ReloadDefaults(void)
 
   monster_infighting = default_monster_infighting; // killough 7/19/98
 
+#ifdef DOGS
   dogs = netgame ? 0 : G_GetHelpers();             // killough 7/19/98
   dog_jumping = default_dog_jumping;
+#endif
 
   distfriend = default_distfriend;                 // killough 8/8/98
 
@@ -2191,6 +2202,11 @@ void G_ReadDemoTiccmd (ticcmd_t* cmd)
 
   if (*demo_p == DEMOMARKER)
     G_CheckDemoStatus();      // end of demo data stream
+  else if (demoplayback && demo_p + (longtics?5:4) > demobuffer + demolength)
+  {
+    lprintf(LO_WARN, "G_ReadDemoTiccmd: missing DEMOMARKER\n");
+    G_CheckDemoStatus();
+  }
   else
     {
       cmd->forwardmove = ((signed char)*demo_p++);
@@ -2261,16 +2277,19 @@ void G_RecordDemo (const char* name)
       int slot = -1;
       int rc;
       int bytes_per_tic;
+      const byte* pos;
 
       { /* Read the demo header for options etc */
         byte buf[200];
-        size_t len;
-        fread(buf, 1, sizeof(buf), demofp);
-      
-        len = G_ReadDemoHeader(buf) - buf;
-        fseek(demofp, len, SEEK_SET);
+        size_t len = fread(buf, 1, sizeof(buf), demofp);
+        pos = G_ReadDemoHeader(buf, len, false);
+        if (pos)
+        {
+          fseek(demofp, pos - buf, SEEK_SET);
+        }
       }
       bytes_per_tic = longtics ? 5 : 4;
+    if (pos)
       /* Now read the demo to find the last save slot */
       do {
         byte buf[5];
@@ -2336,7 +2355,11 @@ byte *G_WriteOptions(byte *demo_p)
 
   *demo_p++ = monster_infighting;   // killough 7/19/98
 
+#ifdef DOGS
   *demo_p++ = dogs;                 // killough 7/19/98
+#else
+  *demo_p++ = 0;
+#endif
 
   *demo_p++ = 0;
   *demo_p++ = 0;
@@ -2352,7 +2375,11 @@ byte *G_WriteOptions(byte *demo_p)
 
   *demo_p++ = help_friends;             // killough 9/9/98
 
+#ifdef DOGS
   *demo_p++ = dog_jumping;
+#else
+  *demo_p++ = 0;
+#endif
 
   *demo_p++ = monkeys;
 
@@ -2422,7 +2449,11 @@ const byte *G_ReadOptions(const byte *demo_p)
     {
       monster_infighting = *demo_p++;   // killough 7/19/98
 
+#ifdef DOGS
       dogs = *demo_p++;                 // killough 7/19/98
+#else
+      demo_p++;
+#endif
 
       demo_p += 2;
 
@@ -2437,7 +2468,11 @@ const byte *G_ReadOptions(const byte *demo_p)
 
       help_friends = *demo_p++;          // killough 9/9/98
 
+#ifdef DOGS
       dog_jumping = *demo_p++;           // killough 10/98
+#else
+      demo_p++;
+#endif
 
       monkeys = *demo_p++;
 
@@ -2603,11 +2638,30 @@ static int G_GetOriginalDoomCompatLevel(int ver)
   return doom2_19_compatibility;
 }
 
-static const byte* G_ReadDemoHeader(const byte *demo_p)
+//e6y: Check for overrun
+static boolean CheckForOverrun(const byte *start_p, const byte *current_p, size_t maxsize, size_t size, boolean failonerror)
+{
+  size_t pos = current_p - start_p;
+  if (pos + size > maxsize)
+  {
+    if (failonerror)
+      I_Error("G_ReadDemoHeader: wrong demo header\n");
+    else
+      return true;
+  }
+  return false;
+}
+
+static const byte* G_ReadDemoHeader(const byte *demo_p, size_t size, boolean failonerror)
 {
   skill_t skill;
   int i, episode, map;
-  int demover;
+
+  // e6y
+  // The local variable should be used instead of demobuffer,
+  // because demobuffer can be uninitialized
+  const byte *header_p = demo_p;
+
   const byte *option_p = NULL;      /* killough 11/98 */
 
   basetic = gametic;  // killough 9/29/98
@@ -2616,9 +2670,25 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
   // Old demos turn on demo_compatibility => compatibility; new demos load
   // compatibility flag, and other flags as well, as a part of the demo.
 
+  //e6y: check for overrun
+  if (CheckForOverrun(header_p, demo_p, size, 1, failonerror))
+    return NULL;
+
   demover = *demo_p++;
   longtics = 0;
 
+  // e6y
+  // Handling of unrecognized demo formats
+  // Versions up to 1.2 use a 7-byte header - first byte is a skill level.
+  // Versions after 1.2 use a 13-byte header - first byte is a demoversion.
+  // BOOM's demoversion starts from 200
+  if (!((demover >=   0  && demover <=   4) ||
+        (demover >= 104  && demover <= 111) ||
+        (demover >= 200  && demover <= 214)))
+  {
+    I_Error("G_ReadDemoHeader: Unknown demo format %d.", demover);
+  }
+
   if (demover < 200)     // Autodetect old demos
     {
       if (demover >= 111) longtics = 1;
@@ -2633,8 +2703,10 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
 
       monster_infighting = 1;           // killough 7/19/98
 
+#ifdef DOGS
       dogs = 0;                         // killough 7/19/98
       dog_jumping = 0;                  // killough 10/98
+#endif
 
       monster_backing = 0;              // killough 9/8/98
 
@@ -2649,7 +2721,11 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
 
       if ((skill=demover) >= 100)         // For demos from versions >= 1.4
         {
-	  compatibility_level = G_GetOriginalDoomCompatLevel(demover);
+          //e6y: check for overrun
+          if (CheckForOverrun(header_p, demo_p, size, 8, failonerror))
+            return NULL;
+
+          compatibility_level = G_GetOriginalDoomCompatLevel(demover);
           skill = *demo_p++;
           episode = *demo_p++;
           map = *demo_p++;
@@ -2661,7 +2737,11 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
         }
       else
         {
-	  compatibility_level = doom_12_compatibility;
+          //e6y: check for overrun
+          if (CheckForOverrun(header_p, demo_p, size, 2, failonerror))
+            return NULL;
+
+          compatibility_level = doom_12_compatibility;
           episode = *demo_p++;
           map = *demo_p++;
           deathmatch = respawnparm = fastparm =
@@ -2675,12 +2755,20 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
       switch (demover) {
       case 200: /* BOOM */
       case 201:
+        //e6y: check for overrun
+        if (CheckForOverrun(header_p, demo_p, size, 1, failonerror))
+          return NULL;
+
         if (!*demo_p++)
 	  compatibility_level = boom_201_compatibility;
         else
 	  compatibility_level = boom_compatibility_compatibility;
 	  break;
       case 202:
+        //e6y: check for overrun
+        if (CheckForOverrun(header_p, demo_p, size, 1, failonerror))
+          return NULL;
+
         if (!*demo_p++)
 	  compatibility_level = boom_202_compatibility;
         else
@@ -2689,7 +2777,7 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
       case 203:
 	/* LxDoom or MBF - determine from signature
 	 * cph - load compatibility level */
-	switch (demobuffer[2]) {
+	switch (*(header_p + 2)) {
 	case 'B': /* LxDoom */
 	  /* cph - DEMOSYNC - LxDoom demos recorded in compatibility modes support dropped */
 	  compatibility_level = lxdoom_1_compatibility;
@@ -2722,6 +2810,10 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
 	demo_p++;
 	break;
       }
+      //e6y: check for overrun
+      if (CheckForOverrun(header_p, demo_p, size, 5, failonerror))
+        return NULL;
+
       skill = *demo_p++;
       episode = *demo_p++;
       map = *demo_p++;
@@ -2732,10 +2824,14 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
       if (mbf_features)
 	option_p = demo_p;
 
+      //e6y: check for overrun
+      if (CheckForOverrun(header_p, demo_p, size, GAME_OPTION_SIZE, failonerror))
+        return NULL;
+
       demo_p = G_ReadOptions(demo_p);  // killough 3/1/98: Read game options
 
       if (demover == 200)              // killough 6/3/98: partially fix v2.00 demos
-        demo_p += 128-GAME_OPTION_SIZE;
+        demo_p += 256-GAME_OPTION_SIZE;
     }
 
   if (sizeof(comp_lev_str)/sizeof(comp_lev_str[0]) != MAX_COMPATIBILITY_LEVEL)
@@ -2745,6 +2841,10 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
 
   if (demo_compatibility)  // only 4 players can exist in old demos
     {
+      //e6y: check for overrun
+      if (CheckForOverrun(header_p, demo_p, size, 4, failonerror))
+        return NULL;
+
       for (i=0; i<4; i++)  // intentionally hard-coded 4 -- killough
         playeringame[i] = *demo_p++;
       for (;i < MAXPLAYERS; i++)
@@ -2752,6 +2852,10 @@ static const byte* G_ReadDemoHeader(const byte *demo_p)
     }
   else
     {
+      //e6y: check for overrun
+      if (CheckForOverrun(header_p, demo_p, size, MAXPLAYERS, failonerror))
+        return NULL;
+
       for (i=0 ; i < MAXPLAYERS; i++)
         playeringame[i] = *demo_p++;
       demo_p += MIN_MAXPLAYERS - MAXPLAYERS;
@@ -2779,28 +2883,21 @@ void G_DoPlayDemo(void)
 
   ExtractFileBase(defdemoname,basename);           // killough
   basename[8] = 0;
-  demobuffer = demo_p = W_CacheLumpNum(demolumpnum = W_GetNumForName(basename));
-  /* cph - store lump number for unlocking later */
 
-  demo_p = G_ReadDemoHeader(demo_p);
+  /* cph - store lump number for unlocking later */
+  demolumpnum = W_GetNumForName(basename);
+  demobuffer = W_CacheLumpNum(demolumpnum);
+  demolength = W_LumpLength(demolumpnum);
+
+  demo_p = G_ReadDemoHeader(demobuffer, demolength, true);
 
   gameaction = ga_nothing;
   usergame = false;
 
   demoplayback = true;
   R_SmoothPlaying_Reset(NULL); // e6y
-}
 
-//
-// G_TimeDemo
-//
-
-void G_TimeDemo(const char *name) // CPhipps - const char*
-{
-  timingdemo = true;
-  singletics = true;
-  defdemoname = name;
-  gameaction = ga_playdemo;
+  starttime = I_GetTime_RealTime ();
 }
 
 /* G_CheckDemoStatus
