@@ -246,7 +246,7 @@ static const float lighttable[5][256] =
   }
 };
 
-#define gld_CalcLightLevel(lightlevel) (lighttable[usegamma][max(min((lightlevel),255),0)])
+#define gld_CalcLightLevel(lightlevel) (lighttable[usegamma][MAX(MIN((lightlevel),255),0)])
 
 /*
 // experimental new lighting code
@@ -348,7 +348,7 @@ void gld_Init(int width, int height)
         p++;
       if (*p)
       {
-        int len = min(p-rover, sizeof(ext_name)-1);
+        int len = MIN(p-rover, sizeof(ext_name)-1);
         memset(ext_name, 0, sizeof(ext_name));
         strncpy(ext_name, rover, len);
         lprintf(LO_INFO,"\t%s\n", ext_name);
@@ -724,9 +724,23 @@ void gld_SetPalette(int palette)
   }
 }
 
-void gld_ReadScreen (byte* scr)
+unsigned char *gld_ReadScreen(void)
 {
-  glReadPixels(0,0,SCREENWIDTH,SCREENHEIGHT,GL_RGB,GL_UNSIGNED_BYTE,scr);
+  unsigned char *scr;
+  unsigned char buffer[MAX_SCREENWIDTH*3];
+  int i;
+
+  scr = malloc(SCREENWIDTH * SCREENHEIGHT * 3);
+  if (scr) {
+    glReadPixels(0,0,SCREENWIDTH,SCREENHEIGHT,GL_RGB,GL_UNSIGNED_BYTE,scr);
+    for (i=0; i<SCREENHEIGHT/2; i++) {
+      memcpy(buffer, &scr[i*SCREENWIDTH*3], SCREENWIDTH*3);
+      memcpy(&scr[i*SCREENWIDTH*3],
+          &scr[(SCREENHEIGHT-(i+1))*SCREENWIDTH*3], SCREENWIDTH*3);
+      memcpy(&scr[(SCREENHEIGHT-(i+1))*SCREENWIDTH*3], buffer, SCREENWIDTH*3);
+    }
+  }
+  return scr;
 }
 
 GLvoid gld_Set2DMode(void)
@@ -1113,7 +1127,9 @@ static void gld_CarveFlats(int bspnode, int numdivlines, divline_t *divlines, bo
   {
     // We have arrived at a subsector. The divline list contains all
     // the partition lines that carve out the subsector.
-    int ssidx = bspnode & (~NF_SUBSECTOR);
+    // special case for trivial maps (no nodes, single subsector)
+    int ssidx = (numnodes != 0) ? bspnode & (~NF_SUBSECTOR) : 0;
+
     if (!sectorclosed[subsectors[ssidx].sector->iSectorID])
       gld_FlatConvexCarver(ssidx, numdivlines, divlines);
     return;
@@ -2016,20 +2032,15 @@ static void gld_DrawWall(GLWall *wall)
     (w).vb=OV((w),(seg))+((float)(lineheight)/(float)(w).gltexture->buffer_height)\
   )
 
-// e6y: about isskytexture hack
-// The sky in the third episode of Requiem was not drawn.
-// It did not work correctly because the SKY3 has a zero index in the TEXTURE1 table.
-// Textures with a zero (FALSE) index are not displayed in vanilla,
-// AASHITTY in doom2.wad for example.
-// But the sky textures are processed by different code in DOOM,
-// which does not have this bug.
+// e6y
+// Sky textures with a zero index should be forced
+// See third episode of requiem.wad
 #define SKYTEXTURE(sky1,sky2)\
-  isskytexture = true;\
   if ((sky1) & PL_SKYFLAT)\
   {\
     const line_t *l = &lines[sky1 & ~PL_SKYFLAT];\
     const side_t *s = *l->sidenum + sides;\
-    wall.gltexture=gld_RegisterTexture(texturetranslation[s->toptexture], false);\
+    wall.gltexture=gld_RegisterTexture(texturetranslation[s->toptexture], false, texturetranslation[s->toptexture]==skytexture);\
     wall.skyyaw=-2.0f*((-(float)((viewangle+s->textureoffset)>>ANGLETOFINESHIFT)*360.0f/FINEANGLES)/90.0f);\
     wall.skyymid = 200.0f/319.5f*(((float)s->rowoffset/(float)FRACUNIT - 28.0f)/100.0f);\
     wall.flag = l->special==272 ? GLDWF_SKY : GLDWF_SKYFLIP;\
@@ -2039,19 +2050,18 @@ static void gld_DrawWall(GLWall *wall)
   {\
     const line_t *l = &lines[sky2 & ~PL_SKYFLAT];\
     const side_t *s = *l->sidenum + sides;\
-    wall.gltexture=gld_RegisterTexture(texturetranslation[s->toptexture], false);\
+    wall.gltexture=gld_RegisterTexture(texturetranslation[s->toptexture], false, texturetranslation[s->toptexture]==skytexture);\
     wall.skyyaw=-2.0f*((-(float)((viewangle+s->textureoffset)>>ANGLETOFINESHIFT)*360.0f/FINEANGLES)/90.0f);\
     wall.skyymid = 200.0f/319.5f*(((float)s->rowoffset/(float)FRACUNIT - 28.0f)/100.0f);\
     wall.flag = l->special==272 ? GLDWF_SKY : GLDWF_SKYFLIP;\
   }\
   else\
   {\
-    wall.gltexture=gld_RegisterTexture(skytexture, false);\
+    wall.gltexture=gld_RegisterTexture(skytexture, false, true);\
     wall.skyyaw=-2.0f*((yaw+90.0f)/90.0f);\
     wall.skyymid = 200.0f/319.5f*((100.0f)/100.0f);\
     wall.flag = GLDWF_SKY;\
-  }\
-  isskytexture = false;
+  };
 
 #define ADDWALL(wall)\
 {\
@@ -2108,7 +2118,7 @@ void gld_AddWall(seg_t *seg)
       SKYTEXTURE(frontsector->sky,frontsector->sky);
       ADDWALL(&wall);
     }
-    temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->midtexture], true);
+    temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->midtexture], true, false);
     if (temptex)
     {
       wall.gltexture=temptex;
@@ -2152,7 +2162,7 @@ void gld_AddWall(seg_t *seg)
           // e6y
           // It corrects some problem with sky, but I do not remember which one
           // old code: wall.ybottom=(float)frontsector->ceilingheight/MAP_SCALE;
-          wall.ybottom=(float)max(frontsector->ceilingheight,backsector->ceilingheight)/MAP_SCALE;
+          wall.ybottom=(float)MAX(frontsector->ceilingheight,backsector->ceilingheight)/MAP_SCALE;
 
           SKYTEXTURE(frontsector->sky,backsector->sky);
           ADDWALL(&wall);
@@ -2171,7 +2181,7 @@ void gld_AddWall(seg_t *seg)
     {
       if (!((frontsector->ceilingpic==skyflatnum) && (backsector->ceilingpic==skyflatnum)))
       {
-        temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->toptexture], true);
+        temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->toptexture], true, false);
         if (temptex)
         {
           wall.gltexture=temptex;
@@ -2186,22 +2196,30 @@ void gld_AddWall(seg_t *seg)
     }
 
     /* midtexture */
-    temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->midtexture], true);
-    if (temptex)
+    //e6y
+    if (comp[comp_maskedanim])
+      temptex=gld_RegisterTexture(seg->sidedef->midtexture, true, false);
+    else
+      // e6y
+      // Animated middle textures with a zero index should be forced
+      // See spacelab.wad (http://www.doomworld.com/idgames/index.php?id=6826)
+      temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->midtexture], true, true);
+
+    if (temptex && seg->sidedef->midtexture != NO_TEXTURE)
     {
       wall.gltexture=temptex;
       if ( (LINE->flags & ML_DONTPEGBOTTOM) >0)
       {
         if (seg->backsector->ceilingheight<=seg->frontsector->floorheight)
           goto bottomtexture;
-        floor_height=max(seg->frontsector->floorheight,seg->backsector->floorheight)+(seg->sidedef->rowoffset);
+        floor_height=MAX(seg->frontsector->floorheight,seg->backsector->floorheight)+(seg->sidedef->rowoffset);
         ceiling_height=floor_height+(wall.gltexture->realtexheight<<FRACBITS);
       }
       else
       {
         if (seg->backsector->ceilingheight<=seg->frontsector->floorheight)
           goto bottomtexture;
-        ceiling_height=min(seg->frontsector->ceilingheight,seg->backsector->ceilingheight)+(seg->sidedef->rowoffset);
+        ceiling_height=MIN(seg->frontsector->ceilingheight,seg->backsector->ceilingheight)+(seg->sidedef->rowoffset);
         floor_height=ceiling_height-(wall.gltexture->realtexheight<<FRACBITS);
       }
       // e6y
@@ -2219,16 +2237,16 @@ void gld_AddWall(seg_t *seg)
         mip = (float)wall.gltexture->realtexheight/(float)wall.gltexture->buffer_height;
 //        if ( (texturetranslation[seg->sidedef->bottomtexture]!=R_TextureNumForName("-")) )
         if (seg->sidedef->bottomtexture)
-          floormax=max(seg->frontsector->floorheight,seg->backsector->floorheight);
+          floormax=MAX(seg->frontsector->floorheight,seg->backsector->floorheight);
         else
           floormax=floor_height;
         if (seg->sidedef->toptexture)
-          ceilingmin=min(seg->frontsector->ceilingheight,seg->backsector->ceilingheight);
+          ceilingmin=MIN(seg->frontsector->ceilingheight,seg->backsector->ceilingheight);
         else
           ceilingmin=ceiling_height;
         linelen=abs(ceiling_height-floor_height);
-        wall.ytop=((float)min(ceilingmin, ceiling_height)/(float)MAP_SCALE);
-        wall.ybottom=((float)max(floormax, floor_height)/(float)MAP_SCALE);
+        wall.ytop=((float)MIN(ceilingmin, ceiling_height)/(float)MAP_SCALE);
+        wall.ybottom=((float)MAX(floormax, floor_height)/(float)MAP_SCALE);
         wall.flag=GLDWF_M2S;
         wall.ul=OU((wall),(seg))+(0.0f);
         wall.ur=OU(wall,(seg))+((segs[seg->iSegID].length)/(float)wall.gltexture->buffer_width);
@@ -2287,7 +2305,7 @@ bottomtexture:
     }
     if (floor_height<ceiling_height)
     {
-      temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->bottomtexture], true);
+      temptex=gld_RegisterTexture(texturetranslation[seg->sidedef->bottomtexture], true, false);
       if (temptex)
       {
         wall.gltexture=temptex;
