@@ -45,7 +45,8 @@ void gld_DrawLine(int x0, int y0, int x1, int y1, int BaseColor);
 void gld_DrawWeapon(int weaponlump, vissprite_t *vis, int lightlevel);
 void gld_FillBlock(int x, int y, int width, int height, int col);
 void gld_SetPalette(int palette);
-void gld_ReadScreen (byte* scr);
+
+unsigned char *gld_ReadScreen (void);
 
 void gld_CleanMemory(void);
 void gld_PreprocessLevel(void);
