@@ -409,21 +409,14 @@ static void gld_AddFlatToTexture(GLTexture *gltexture, unsigned char *buffer, co
   }
 }
 
-GLTexture *gld_RegisterTexture(int texture_num, boolean mipmap)
+//e6y: "force" flag for loading texture with zero index
+GLTexture *gld_RegisterTexture(int texture_num, boolean mipmap, boolean force)
 {
   GLTexture *gltexture;
 
-  if (texture_num==NO_TEXTURE)
-// e6y: about isskytexture hack
-// The sky in the third episode of Requiem was not drawn.
-// It did not work correctly because the SKY3 has a zero index in the TEXTURE1 table.
-// Textures with a zero (FALSE) index are not displayed in vanilla,
-// AASHITTY in doom2.wad for example.
-// But the sky textures are processed by different code in DOOM,
-// which does not have this bug.
-// This flag is set in SKYTEXTURE macro and checked here for skipping unnecessary return
-    if (!isskytexture || texture_num!=skytexture)
-      return NULL;
+  //e6y: textures with zero index should be loaded sometimes
+  if (texture_num==NO_TEXTURE && !force)
+    return NULL;
   gltexture=gld_AddNewGLTexture(texture_num);
   if (!gltexture)
     return NULL;
@@ -444,8 +437,8 @@ GLTexture *gld_RegisterTexture(int texture_num, boolean mipmap)
     gltexture->topoffset=0;
     gltexture->tex_width=gld_GetTexDimension(gltexture->realtexwidth);
     gltexture->tex_height=gld_GetTexDimension(gltexture->realtexheight);
-    gltexture->width=min(gltexture->realtexwidth, gltexture->tex_width);
-    gltexture->height=min(gltexture->realtexheight, gltexture->tex_height);
+    gltexture->width=MIN(gltexture->realtexwidth, gltexture->tex_width);
+    gltexture->height=MIN(gltexture->realtexheight, gltexture->tex_height);
     gltexture->buffer_width=gltexture->tex_width;
     gltexture->buffer_height=gltexture->tex_height;
 #ifdef USE_GLU_IMAGESCALE
@@ -599,15 +592,15 @@ GLTexture *gld_RegisterPatch(int lump, int cm)
     gltexture->topoffset=patch->topoffset;
     gltexture->tex_width=gld_GetTexDimension(gltexture->realtexwidth);
     gltexture->tex_height=gld_GetTexDimension(gltexture->realtexheight);
-    gltexture->width=min(gltexture->realtexwidth, gltexture->tex_width);
-    gltexture->height=min(gltexture->realtexheight, gltexture->tex_height);
+    gltexture->width=MIN(gltexture->realtexwidth, gltexture->tex_width);
+    gltexture->height=MIN(gltexture->realtexheight, gltexture->tex_height);
     gltexture->buffer_width=gltexture->tex_width;
     gltexture->buffer_height=gltexture->tex_height;
 #ifdef USE_GLU_IMAGESCALE
-    gltexture->width=min(gltexture->realtexwidth, gltexture->tex_width);
-    gltexture->height=min(gltexture->realtexheight, gltexture->tex_height);
-    gltexture->buffer_width=max(gltexture->realtexwidth, gltexture->tex_width);
-    gltexture->buffer_height=max(gltexture->realtexheight, gltexture->tex_height);
+    gltexture->width=MIN(gltexture->realtexwidth, gltexture->tex_width);
+    gltexture->height=MIN(gltexture->realtexheight, gltexture->tex_height);
+    gltexture->buffer_width=MAX(gltexture->realtexwidth, gltexture->tex_width);
+    gltexture->buffer_height=MAX(gltexture->realtexheight, gltexture->tex_height);
 #endif
     gltexture->buffer_size=gltexture->buffer_width*gltexture->buffer_height*4;
     R_UnlockPatchNum(lump);
@@ -722,8 +715,8 @@ GLTexture *gld_RegisterFlat(int lump, boolean mipmap)
     gltexture->topoffset=0;
     gltexture->tex_width=gld_GetTexDimension(gltexture->realtexwidth);
     gltexture->tex_height=gld_GetTexDimension(gltexture->realtexheight);
-    gltexture->width=min(gltexture->realtexwidth, gltexture->tex_width);
-    gltexture->height=min(gltexture->realtexheight, gltexture->tex_height);
+    gltexture->width=MIN(gltexture->realtexwidth, gltexture->tex_width);
+    gltexture->height=MIN(gltexture->realtexheight, gltexture->tex_height);
     gltexture->buffer_width=gltexture->tex_width;
     gltexture->buffer_height=gltexture->tex_height;
 #ifdef USE_GLU_IMAGESCALE
@@ -929,7 +922,7 @@ void gld_Precache(void)
 
   for (i = numtextures; --i >= 0; )
     if (hitlist[i])
-      gld_BindTexture(gld_RegisterTexture(i,true));
+      gld_BindTexture(gld_RegisterTexture(i,true,false));
 
   // Precache sprites.
   memset(hitlist, 0, numsprites);
