@@ -444,6 +444,7 @@ static boolean P_SmartMove(mobj_t *actor)
   // dropoff==1 means always allow it, dropoff==2 means only up to 128 high,
   // and only if the target is immediately on the other side of the line.
 
+#ifdef DOGS
   // haleyjd: allow all friends of HelperType to also jump down
 
   if ((actor->type == MT_DOGS || (actor->type == (HelperThing-1) && actor->flags&MF_FRIEND))
@@ -453,6 +454,7 @@ static boolean P_SmartMove(mobj_t *actor)
           actor->y - target->y) < FRACUNIT*144 &&
       P_Random(pr_dropoff) < 235)
     dropoff = 2;
+#endif
 
   if (!P_Move(actor, dropoff))
     return false;
@@ -825,7 +827,24 @@ static boolean P_LookForPlayers(mobj_t *actor, boolean allaround)
 
       // killough 2/15/98, 9/9/98:
       if (c++ == stopc || actor->lastlook == stop)  // done looking
-  return false;
+      {
+        // e6y
+        // Fixed Boom incompatibilities. The following code was missed.
+        // There are no more desyncs on Donce's demos on horror.wad
+
+        // Use last known enemy if no players sighted -- killough 2/15/98:
+        if (!mbf_features && !demo_compatibility && monsters_remember)
+        {
+          if (actor->lastenemy && actor->lastenemy->health > 0)
+          {
+            actor->target = actor->lastenemy;
+            actor->lastenemy = NULL;
+            return true;
+          }
+        }
+
+        return false;
+      }
 
       player = &players[actor->lastlook];
 
@@ -1819,6 +1838,9 @@ void A_FatAttack1(mobj_t *actor)
   mobj_t *mo;
   int    an;
 
+  if (!actor->target)
+    return;
+
   A_FaceTarget(actor);
 
   // Change direction  to ...
@@ -1838,6 +1860,9 @@ void A_FatAttack2(mobj_t *actor)
   mobj_t *mo;
   int    an;
 
+  if (!actor->target)
+    return;
+
   A_FaceTarget(actor);
   // Now here choose opposite deviation.
   actor->angle -= FATSPREAD;
@@ -1855,6 +1880,9 @@ void A_FatAttack3(mobj_t *actor)
   mobj_t *mo;
   int    an;
 
+  if (!actor->target)
+    return;
+
   A_FaceTarget(actor);
 
   mo = P_SpawnMissile(actor, actor->target, MT_FATSHOT);
@@ -2090,21 +2118,39 @@ void A_BossDeath(mobj_t *mo)
     }
   else
     {
+      // e6y
+      // Additional check of gameepisode is necessary, because 
+      // there is no right or wrong solution for E4M6 in original EXEs,
+      // there's nothing to emulate.
+      if (comp[comp_666] && gameepisode < 4)
+      {
+        // e6y
+        // Only following checks are present in doom2.exe ver. 1.666 and 1.9
+        // instead of separate checks for each episode in doomult.exe, plutonia.exe and tnt.exe
+        // There is no more desync on doom.wad\episode3.lmp
+        // http://www.doomworld.com/idgames/index.php?id=6909
+        if (gamemap != 8)
+          return;
+        if (mo->type == MT_BRUISER && gameepisode != 1) 
+          return;
+      }
+      else
+      {
       switch(gameepisode)
         {
         case 1:
           if (gamemap != 8)
             return;
 
-	  if (mo->type != MT_BRUISER && !comp[comp_666])
-	    return;
+          if (mo->type != MT_BRUISER)
+            return;
           break;
 
         case 2:
           if (gamemap != 8)
             return;
 
-          if (mo->type != MT_CYBORG && !comp[comp_666])
+          if (mo->type != MT_CYBORG)
             return;
           break;
 
@@ -2112,7 +2158,7 @@ void A_BossDeath(mobj_t *mo)
           if (gamemap != 8)
             return;
 
-          if (mo->type != MT_SPIDER && !comp[comp_666])
+          if (mo->type != MT_SPIDER)
             return;
 
           break;
@@ -2141,6 +2187,7 @@ void A_BossDeath(mobj_t *mo)
             return;
           break;
         }
+      }
 
     }
 
