@@ -187,7 +187,8 @@ static boolean P_GiveWeapon(player_t *player, weapontype_t weapon, boolean dropp
       player->pendingweapon = weapon;
       /* cph 20028/10 - for old-school DM addicts, allow old behavior
        * where only consoleplayer's pickup sounds are heard */
-      if (!comp[comp_sound] || player == &players[consoleplayer])
+      // displayplayer, not consoleplayer, for viewing multiplayer demos
+      if (!comp[comp_sound] || player == &players[displayplayer])
         S_StartSound (player->mo, sfx_wpnup|PICKUP_SOUND); // killough 4/25/98
       return false;
     }
@@ -608,7 +609,8 @@ void P_TouchSpecialThing(mobj_t *special, mobj_t *toucher)
 
   /* cph 20028/10 - for old-school DM addicts, allow old behavior
    * where only consoleplayer's pickup sounds are heard */
-  if (!comp[comp_sound] || player == &players[consoleplayer])
+  // displayplayer, not consoleplayer, for viewing multiplayer demos
+  if (!comp[comp_sound] || player == &players[displayplayer])
     S_StartSound (player->mo, sound | PICKUP_SOUND);   // killough 4/25/98
 }
 
