@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -64,19 +66,17 @@ typedef struct {
 typedef boolean (*traverser_t)(intercept_t *in);
 
 fixed_t CONSTFUNC P_AproxDistance (fixed_t dx, fixed_t dy);
-int     CONSTFUNC P_PointOnLineSide (fixed_t x, fixed_t y, const line_t *line);
-int     CONSTFUNC P_PointOnDivlineSide (fixed_t x, fixed_t y,
-          const divline_t *line);
-int     CONSTFUNC P_BoxOnLineSide (const fixed_t *tmbox, const line_t *ld);
-fixed_t CONSTFUNC P_InterceptVector (const divline_t *v2, const divline_t *v1);
+int     PUREFUNC  P_PointOnLineSide (fixed_t x, fixed_t y, const line_t *line);
+int     PUREFUNC  P_BoxOnLineSide (const fixed_t *tmbox, const line_t *ld);
+fixed_t PUREFUNC  P_InterceptVector (const divline_t *v2, const divline_t *v1);
+/* cph - old compatibility version below */
+fixed_t PUREFUNC  P_InterceptVector2(const divline_t *v2, const divline_t *v1);
 
-void    P_MakeDivline (const line_t *li, divline_t *dl);
 void    P_LineOpening (const line_t *linedef);
 void    P_UnsetThingPosition(mobj_t *thing);
 void    P_SetThingPosition(mobj_t *thing);
 boolean P_BlockLinesIterator (int x, int y, boolean func(line_t *));
 boolean P_BlockThingsIterator(int x, int y, boolean func(mobj_t *));
-boolean CONSTFUNC ThingIsOnLine(const mobj_t *t, const line_t *l);  /* killough 3/15/98 */
 boolean P_PathTraverse(fixed_t x1, fixed_t y1, fixed_t x2, fixed_t y2,
                        int flags, boolean trav(intercept_t *));
 
