@@ -339,6 +339,10 @@ typedef struct mobj_s
 
     // killough 8/2/98: friction properties part of sectors,
     // not objects -- removed friction properties from here
+    // e6y: restored friction properties here
+    // Friction values for the sector the object is in
+    int friction;                                           // phares 3/17/98
+    int movefactor;
 
     // a linked list of sectors where this object appears
     struct msecnode_s* touching_sectorlist;                 // phares 3/14/98
