@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -40,6 +42,7 @@
 #include "s_sound.h"
 #include "sounds.h"
 #include "d_event.h"
+#include "r_demo.h"
 
 #define LOWERSPEED   (FRACUNIT*6)
 #define RAISESPEED   (FRACUNIT*6)
@@ -268,8 +271,6 @@ boolean P_CheckAmmo(player_t *player)
 // P_FireWeapon.
 //
 
-int lastshottic; // killough 3/22/98
-
 static void P_FireWeapon(player_t *player)
 {
   statenum_t newstate;
@@ -281,7 +282,6 @@ static void P_FireWeapon(player_t *player)
   newstate = weaponinfo[player->readyweapon].atkstate;
   P_SetPsprite(player, ps_weapon, newstate);
   P_NoiseAlert(player->mo, player->mo);
-  lastshottic = gametic;                       // killough 3/22/98
 }
 
 //
@@ -513,6 +513,7 @@ void A_Punch(player_t *player, pspdef_t *psp)
 
   player->mo->angle = R_PointToAngle2(player->mo->x, player->mo->y,
                                       linetarget->x, linetarget->y);
+  R_SmoothPlaying_Reset(player); // e6y
 }
 
 //
@@ -561,6 +562,7 @@ void A_Saw(player_t *player, pspdef_t *psp)
   }
 
   player->mo->flags |= MF_JUSTATTACKED;
+  R_SmoothPlaying_Reset(player); // e6y
 }
 
 //
@@ -583,20 +585,6 @@ void A_FireBFG(player_t *player, pspdef_t *psp)
   P_SpawnPlayerMissile(player->mo, MT_BFG);
 }
 
-/*
- * A_FireOldBFG
- *
- * This function emulates Doom's Pre-Beta BFG
- * By Lee Killough 6/6/98, 7/11/98, 7/19/98, 8/20/98
- *
- * This code may not be used in other mods without appropriate credit given.
- * Code leeches will be telefragged.
- */
-
-void A_FireOldBFG(player_t *player, pspdef_t *psp)
-{
-}
-
 //
 // A_FirePlasma
 //
@@ -639,7 +627,7 @@ static void P_BulletSlope(mobj_t *mo)
 // P_GunShot
 //
 
-void P_GunShot(mobj_t *mo, boolean accurate)
+static void P_GunShot(mobj_t *mo, boolean accurate)
 {
   int damage = 5*(P_Random(pr_gunshot)%3+1);
   angle_t angle = mo->angle;
