@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -31,6 +33,7 @@
 
 #include "doomstat.h"
 #include "r_main.h"
+#include "p_map.h"
 #include "p_maputl.h"
 #include "p_setup.h"
 #include "m_bbox.h"
@@ -69,21 +72,6 @@ inline static int P_DivlineSide(fixed_t x, fixed_t y, const divline_t *node)
     right == left ? 2 : 1;
 }
 
-//
-// P_InterceptVector2
-// Returns the fractional intercept point
-// along the first divline.
-//
-// killough 4/19/98: made static, cleaned up
-
-static fixed_t P_InterceptVector2(const divline_t *v2, const divline_t *v1)
-{
-  fixed_t den;
-  return (den = FixedMul(v1->dy>>8, v2->dx) - FixedMul(v1->dx>>8, v2->dy)) ?
-    FixedDiv(FixedMul((v1->x - v2->x)>>8, v1->dy) +
-             FixedMul((v2->y - v1->y)>>8, v1->dx), den) : 0;
-}
-
 //
 // P_CrossSubsector
 // Returns true
@@ -176,14 +164,17 @@ static boolean P_CrossSubsector(int num)
   return false;
 
     { // crosses a two sided line
-      fixed_t frac = P_InterceptVector2(&los.strace, &divl);
+      /* cph 2006/07/15 - oops, we missed this in 2.4.0 & .1;
+       *  use P_InterceptVector2 for those compat levels only. */ 
+      fixed_t frac = (compatibility_level == prboom_5_compatibility || compatibility_level == prboom_6_compatibility) ?
+		      P_InterceptVector2(&los.strace, &divl) : 
+		      P_InterceptVector(&los.strace, &divl);
 
-      if (front->floorheight != back->floorheight)
-  {
-    fixed_t slope = FixedDiv(openbottom - los.sightzstart , frac);
-    if (slope > los.bottomslope)
+      if (front->floorheight != back->floorheight) {
+        fixed_t slope = FixedDiv(openbottom - los.sightzstart , frac);
+        if (slope > los.bottomslope)
             los.bottomslope = slope;
-  }
+      }
 
       if (front->ceilingheight != back->ceilingheight)
         {
@@ -235,8 +226,8 @@ static boolean P_CrossBSPNode_PrBoom(int bspnum)
     {
       register const node_t *bsp = nodes + bspnum;
       int side,side2;
-      side = P_DivlineSide(los.strace.x,los.strace.y,(divline_t *)bsp)&1;
-      side2= P_DivlineSide(los.t2x, los.t2y, (divline_t *) bsp);
+      side = P_DivlineSide(los.strace.x,los.strace.y,(const divline_t *)bsp)&1;
+      side2= P_DivlineSide(los.t2x, los.t2y, (const divline_t *) bsp);
       if (side == side2)
          bspnum = bsp->children[side]; // doesn't touch the other side
       else         // the partition plane is crossed here
