@@ -87,7 +87,9 @@ typedef enum
 
 static int    temp_x = 0;
 static int    tempyl[4], tempyh[4];
-static byte   tempbuf[MAX_SCREENHEIGHT * 4];
+static byte           byte_tempbuf[MAX_SCREENHEIGHT * 4];
+static unsigned short short_tempbuf[MAX_SCREENHEIGHT * 4];
+static unsigned int   int_tempbuf[MAX_SCREENHEIGHT * 4];
 static int    startx = 0;
 static int    temptype = COL_NONE;
 static int    commontop, commonbot;
@@ -131,8 +133,12 @@ static int fuzzpos = 0;
 #define RDC_ROUNDED     128
 
 draw_vars_t drawvars = { 
-  NULL, // topleft
-  0, // pitch
+  NULL, // byte_topleft
+  NULL, // short_topleft
+  NULL, // int_topleft
+  0, // byte_pitch
+  0, // short_pitch
+  0, // int_pitch
   RDRAW_FILTER_POINT, // filterwall
   RDRAW_FILTER_POINT, // filterfloor
   RDRAW_FILTER_POINT, // filtersprite
@@ -202,23 +208,89 @@ void R_ResetColumnBuffer(void)
 }
 
 #define R_DRAWCOLUMN_PIPELINE RDC_STANDARD
+#define R_DRAWCOLUMN_PIPELINE_BITS 8
 #define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
 #define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
 #define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
 #include "r_drawflush.inl"
 
 #define R_DRAWCOLUMN_PIPELINE RDC_TRANSLUCENT
+#define R_DRAWCOLUMN_PIPELINE_BITS 8
 #define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
 #define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
 #define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
 #include "r_drawflush.inl"
 
 #define R_DRAWCOLUMN_PIPELINE RDC_FUZZ
+#define R_DRAWCOLUMN_PIPELINE_BITS 8
 #define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
 #define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
 #define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
 #include "r_drawflush.inl"
 
+#define R_DRAWCOLUMN_PIPELINE RDC_STANDARD
+#define R_DRAWCOLUMN_PIPELINE_BITS 15
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole15
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT15
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad15
+#include "r_drawflush.inl"
+
+#define R_DRAWCOLUMN_PIPELINE RDC_TRANSLUCENT
+#define R_DRAWCOLUMN_PIPELINE_BITS 15
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL15
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL15
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL15
+#include "r_drawflush.inl"
+
+#define R_DRAWCOLUMN_PIPELINE RDC_FUZZ
+#define R_DRAWCOLUMN_PIPELINE_BITS 15
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz15
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz15
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz15
+#include "r_drawflush.inl"
+
+#define R_DRAWCOLUMN_PIPELINE RDC_STANDARD
+#define R_DRAWCOLUMN_PIPELINE_BITS 16
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole16
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT16
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad16
+#include "r_drawflush.inl"
+
+#define R_DRAWCOLUMN_PIPELINE RDC_TRANSLUCENT
+#define R_DRAWCOLUMN_PIPELINE_BITS 16
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL16
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL16
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL16
+#include "r_drawflush.inl"
+
+#define R_DRAWCOLUMN_PIPELINE RDC_FUZZ
+#define R_DRAWCOLUMN_PIPELINE_BITS 16
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz16
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz16
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz16
+#include "r_drawflush.inl"
+
+#define R_DRAWCOLUMN_PIPELINE RDC_STANDARD
+#define R_DRAWCOLUMN_PIPELINE_BITS 32
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole32
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT32
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad32
+#include "r_drawflush.inl"
+
+#define R_DRAWCOLUMN_PIPELINE RDC_TRANSLUCENT
+#define R_DRAWCOLUMN_PIPELINE_BITS 32
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL32
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL32
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL32
+#include "r_drawflush.inl"
+
+#define R_DRAWCOLUMN_PIPELINE RDC_FUZZ
+#define R_DRAWCOLUMN_PIPELINE_BITS 32
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz32
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz32
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz32
+#include "r_drawflush.inl"
+
 //
 // R_DrawColumn
 //
@@ -234,79 +306,37 @@ void R_ResetColumnBuffer(void)
 byte *translationtables;
 
 #define R_DRAWCOLUMN_PIPELINE_TYPE RDC_PIPELINE_STANDARD
+#define R_DRAWCOLUMN_PIPELINE_BASE RDC_STANDARD
 
-// standard with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_PointUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_STANDARD | RDC_NOCOLMAP)
+#define R_DRAWCOLUMN_PIPELINE_BITS 8
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawColumn8 ## postfix
 #define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
 #define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
 #define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
+#include "r_drawcolpipeline.inl"
 
-// standard
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_PointUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE RDC_STANDARD
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 15
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawColumn15 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole15
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT15
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad15
+#include "r_drawcolpipeline.inl"
 
-// standard + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_PointUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_STANDARD | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 16
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawColumn16 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole16
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT16
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad16
+#include "r_drawcolpipeline.inl"
 
-// standard + bilinear with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_LinearUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_STANDARD | RDC_BILINEAR | RDC_NOCOLMAP)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// standard + bilinear
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_LinearUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_STANDARD | RDC_BILINEAR)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// standard + bilinear + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_LinearUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_STANDARD | RDC_BILINEAR | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// standard + rounded with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_RoundedUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_STANDARD | RDC_ROUNDED | RDC_NOCOLMAP)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// standard + rounded
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_RoundedUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_STANDARD | RDC_ROUNDED)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// standard + rounded + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawColumn8_RoundedUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_STANDARD | RDC_ROUNDED | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 32
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawColumn32 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole32
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT32
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad32
+#include "r_drawcolpipeline.inl"
 
+#undef R_DRAWCOLUMN_PIPELINE_BASE
 #undef R_DRAWCOLUMN_PIPELINE_TYPE
 
 // Here is the version of R_DrawColumn that deals with translucent  // phares
@@ -322,79 +352,37 @@ byte *translationtables;
 // actual code differences are.
 
 #define R_DRAWCOLUMN_PIPELINE_TYPE RDC_PIPELINE_TRANSLUCENT
+#define R_DRAWCOLUMN_PIPELINE_BASE RDC_TRANSLUCENT
 
-// translucent with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_PointUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLUCENT | RDC_NOCOLMAP)
+#define R_DRAWCOLUMN_PIPELINE_BITS 8
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawTLColumn8 ## postfix
 #define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
 #define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
 #define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
+#include "r_drawcolpipeline.inl"
 
-// translucent
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_PointUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE RDC_TRANSLUCENT
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 15
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawTLColumn15 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL15
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL15
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL15
+#include "r_drawcolpipeline.inl"
 
-// translucent + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_PointUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLUCENT | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 16
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawTLColumn16 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL16
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL16
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL16
+#include "r_drawcolpipeline.inl"
 
-// translucent + bilinear with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_LinearUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLUCENT | RDC_BILINEAR | RDC_NOCOLMAP)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
-
-// translucent + bilinear
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_LinearUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLUCENT | RDC_BILINEAR)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
-
-// translucent + bilinear + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_LinearUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLUCENT | RDC_BILINEAR | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
-
-// translucent + rounded with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_RoundedUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLUCENT | RDC_ROUNDED | RDC_NOCOLMAP)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
-
-// translucent + rounded
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_RoundedUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLUCENT | RDC_ROUNDED)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
-
-// translucent + rounded + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTLColumn8_RoundedUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLUCENT | RDC_ROUNDED | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 32
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawTLColumn32 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeTL32
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTTL32
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadTL32
+#include "r_drawcolpipeline.inl"
 
+#undef R_DRAWCOLUMN_PIPELINE_BASE
 #undef R_DRAWCOLUMN_PIPELINE_TYPE
 
 //
@@ -408,79 +396,37 @@ byte *translationtables;
 //
 
 #define R_DRAWCOLUMN_PIPELINE_TYPE RDC_PIPELINE_TRANSLATED
+#define R_DRAWCOLUMN_PIPELINE_BASE RDC_TRANSLATED
 
-// translated with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_PointUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLATED | RDC_NOCOLMAP)
+#define R_DRAWCOLUMN_PIPELINE_BITS 8
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawTranslatedColumn8 ## postfix
 #define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
 #define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
 #define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
+#include "r_drawcolpipeline.inl"
 
-// translated
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_PointUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE RDC_TRANSLATED
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 15
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawTranslatedColumn15 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole15
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT15
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad15
+#include "r_drawcolpipeline.inl"
 
-// translated + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_PointUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLATED | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 16
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawTranslatedColumn16 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole16
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT16
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad16
+#include "r_drawcolpipeline.inl"
 
-// translated + bilinear with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_LinearUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLATED | RDC_BILINEAR | RDC_NOCOLMAP)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// translated + bilinear
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_LinearUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLATED | RDC_BILINEAR)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// translated + bilinear + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_LinearUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLATED | RDC_BILINEAR | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// translated + rounded with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_RoundedUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLATED | RDC_ROUNDED | RDC_NOCOLMAP)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// translated + rounded
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_RoundedUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLATED | RDC_ROUNDED)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
-
-// translated + rounded + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawTranslatedColumn8_RoundedUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_TRANSLATED | RDC_ROUNDED | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuad8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 32
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawTranslatedColumn32 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWhole32
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHT32
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuad32
+#include "r_drawcolpipeline.inl"
 
+#undef R_DRAWCOLUMN_PIPELINE_BASE
 #undef R_DRAWCOLUMN_PIPELINE_TYPE
 
 //
@@ -493,133 +439,234 @@ byte *translationtables;
 //
 
 #define R_DRAWCOLUMN_PIPELINE_TYPE RDC_PIPELINE_FUZZ
+#define R_DRAWCOLUMN_PIPELINE_BASE RDC_FUZZ
 
-// fuzz with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_PointUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_FUZZ | RDC_NOCOLMAP)
+#define R_DRAWCOLUMN_PIPELINE_BITS 8
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawFuzzColumn8 ## postfix
 #define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
 #define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
 #define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
+#include "r_drawcolpipeline.inl"
 
-// fuzz
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_PointUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE RDC_FUZZ
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 15
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawFuzzColumn15 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz15
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz15
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz15
+#include "r_drawcolpipeline.inl"
 
-// fuzz + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_PointUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_FUZZ | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 16
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawFuzzColumn16 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz16
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz16
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz16
+#include "r_drawcolpipeline.inl"
 
-// fuzz + bilinear with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_LinearUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_FUZZ | RDC_BILINEAR | RDC_NOCOLMAP)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
-
-// fuzz + bilinear
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_LinearUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_FUZZ | RDC_BILINEAR)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
-
-// fuzz + bilinear + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_LinearUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_FUZZ | RDC_BILINEAR | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
-
-// fuzz + rounded with no color mapping
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_RoundedUV
-#define R_DRAWCOLUMN_PIPELINE (RDC_FUZZ | RDC_ROUNDED | RDC_NOCOLMAP)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
-
-// fuzz + rounded
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_RoundedUV_PointZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_FUZZ | RDC_ROUNDED)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
-
-// fuzz + rounded + z-dither
-#define R_DRAWCOLUMN_FUNCNAME R_DrawFuzzColumn8_RoundedUV_LinearZ
-#define R_DRAWCOLUMN_PIPELINE (RDC_FUZZ | RDC_ROUNDED | RDC_DITHERZ)
-#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz8
-#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz8
-#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz8
-#include "r_drawcolumn.inl"
+#define R_DRAWCOLUMN_PIPELINE_BITS 32
+#define R_DRAWCOLUMN_FUNCNAME_COMPOSITE(postfix) R_DrawFuzzColumn32 ## postfix
+#define R_FLUSHWHOLE_FUNCNAME R_FlushWholeFuzz32
+#define R_FLUSHHEADTAIL_FUNCNAME R_FlushHTFuzz32
+#define R_FLUSHQUAD_FUNCNAME R_FlushQuadFuzz32
+#include "r_drawcolpipeline.inl"
 
+#undef R_DRAWCOLUMN_PIPELINE_BASE
 #undef R_DRAWCOLUMN_PIPELINE_TYPE
 
-static R_DrawColumn_f drawcolumnfuncs[RDRAW_FILTER_MAXFILTERS][RDRAW_FILTER_MAXFILTERS][RDC_PIPELINE_MAXPIPELINES] = {
+static R_DrawColumn_f drawcolumnfuncs[VID_MODEMAX][RDRAW_FILTER_MAXFILTERS][RDRAW_FILTER_MAXFILTERS][RDC_PIPELINE_MAXPIPELINES] = {
   {
-    {NULL, NULL, NULL, NULL,},
-    {R_DrawColumn8_PointUV,
-     R_DrawTLColumn8_PointUV,
-     R_DrawTranslatedColumn8_PointUV,
-     R_DrawFuzzColumn8_PointUV,},
-    {R_DrawColumn8_LinearUV,
-     R_DrawTLColumn8_LinearUV,
-     R_DrawTranslatedColumn8_LinearUV,
-     R_DrawFuzzColumn8_LinearUV,},
-    {R_DrawColumn8_RoundedUV,
-     R_DrawTLColumn8_RoundedUV,
-     R_DrawTranslatedColumn8_RoundedUV,
-     R_DrawFuzzColumn8_RoundedUV,},
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn8_PointUV,
+       R_DrawTLColumn8_PointUV,
+       R_DrawTranslatedColumn8_PointUV,
+       R_DrawFuzzColumn8_PointUV,},
+      {R_DrawColumn8_LinearUV,
+       R_DrawTLColumn8_LinearUV,
+       R_DrawTranslatedColumn8_LinearUV,
+       R_DrawFuzzColumn8_LinearUV,},
+      {R_DrawColumn8_RoundedUV,
+       R_DrawTLColumn8_RoundedUV,
+       R_DrawTranslatedColumn8_RoundedUV,
+       R_DrawFuzzColumn8_RoundedUV,},
+    },
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn8_PointUV_PointZ,
+       R_DrawTLColumn8_PointUV_PointZ,
+       R_DrawTranslatedColumn8_PointUV_PointZ,
+       R_DrawFuzzColumn8_PointUV_PointZ,},
+      {R_DrawColumn8_LinearUV_PointZ,
+       R_DrawTLColumn8_LinearUV_PointZ,
+       R_DrawTranslatedColumn8_LinearUV_PointZ,
+       R_DrawFuzzColumn8_LinearUV_PointZ,},
+      {R_DrawColumn8_RoundedUV_PointZ,
+       R_DrawTLColumn8_RoundedUV_PointZ,
+       R_DrawTranslatedColumn8_RoundedUV_PointZ,
+       R_DrawFuzzColumn8_RoundedUV_PointZ,},
+    },
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn8_PointUV_LinearZ,
+       R_DrawTLColumn8_PointUV_LinearZ,
+       R_DrawTranslatedColumn8_PointUV_LinearZ,
+       R_DrawFuzzColumn8_PointUV_LinearZ,},
+      {R_DrawColumn8_LinearUV_LinearZ,
+       R_DrawTLColumn8_LinearUV_LinearZ,
+       R_DrawTranslatedColumn8_LinearUV_LinearZ,
+       R_DrawFuzzColumn8_LinearUV_LinearZ,},
+      {R_DrawColumn8_RoundedUV_LinearZ,
+       R_DrawTLColumn8_RoundedUV_LinearZ,
+       R_DrawTranslatedColumn8_RoundedUV_LinearZ,
+       R_DrawFuzzColumn8_RoundedUV_LinearZ,},
+    },
   },
   {
-    {NULL, NULL, NULL, NULL,},
-    {R_DrawColumn8_PointUV_PointZ,
-     R_DrawTLColumn8_PointUV_PointZ,
-     R_DrawTranslatedColumn8_PointUV_PointZ,
-     R_DrawFuzzColumn8_PointUV_PointZ,},
-    {R_DrawColumn8_LinearUV_PointZ,
-     R_DrawTLColumn8_LinearUV_PointZ,
-     R_DrawTranslatedColumn8_LinearUV_PointZ,
-     R_DrawFuzzColumn8_LinearUV_PointZ,},
-    {R_DrawColumn8_RoundedUV_PointZ,
-     R_DrawTLColumn8_RoundedUV_PointZ,
-     R_DrawTranslatedColumn8_RoundedUV_PointZ,
-     R_DrawFuzzColumn8_RoundedUV_PointZ,},
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn15_PointUV,
+       R_DrawTLColumn15_PointUV,
+       R_DrawTranslatedColumn15_PointUV,
+       R_DrawFuzzColumn15_PointUV,},
+      {R_DrawColumn15_LinearUV,
+       R_DrawTLColumn15_LinearUV,
+       R_DrawTranslatedColumn15_LinearUV,
+       R_DrawFuzzColumn15_LinearUV,},
+      {R_DrawColumn15_RoundedUV,
+       R_DrawTLColumn15_RoundedUV,
+       R_DrawTranslatedColumn15_RoundedUV,
+       R_DrawFuzzColumn15_RoundedUV,},
+    },
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn15_PointUV_PointZ,
+       R_DrawTLColumn15_PointUV_PointZ,
+       R_DrawTranslatedColumn15_PointUV_PointZ,
+       R_DrawFuzzColumn15_PointUV_PointZ,},
+      {R_DrawColumn15_LinearUV_PointZ,
+       R_DrawTLColumn15_LinearUV_PointZ,
+       R_DrawTranslatedColumn15_LinearUV_PointZ,
+       R_DrawFuzzColumn15_LinearUV_PointZ,},
+      {R_DrawColumn15_RoundedUV_PointZ,
+       R_DrawTLColumn15_RoundedUV_PointZ,
+       R_DrawTranslatedColumn15_RoundedUV_PointZ,
+       R_DrawFuzzColumn15_RoundedUV_PointZ,},
+    },
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn15_PointUV_LinearZ,
+       R_DrawTLColumn15_PointUV_LinearZ,
+       R_DrawTranslatedColumn15_PointUV_LinearZ,
+       R_DrawFuzzColumn15_PointUV_LinearZ,},
+      {R_DrawColumn15_LinearUV_LinearZ,
+       R_DrawTLColumn15_LinearUV_LinearZ,
+       R_DrawTranslatedColumn15_LinearUV_LinearZ,
+       R_DrawFuzzColumn15_LinearUV_LinearZ,},
+      {R_DrawColumn15_RoundedUV_LinearZ,
+       R_DrawTLColumn15_RoundedUV_LinearZ,
+       R_DrawTranslatedColumn15_RoundedUV_LinearZ,
+       R_DrawFuzzColumn15_RoundedUV_LinearZ,},
+    },
   },
   {
-    {NULL, NULL, NULL, NULL,},
-    {R_DrawColumn8_PointUV_LinearZ,
-     R_DrawTLColumn8_PointUV_LinearZ,
-     R_DrawTranslatedColumn8_PointUV_LinearZ,
-     R_DrawFuzzColumn8_PointUV_LinearZ,},
-    {R_DrawColumn8_LinearUV_LinearZ,
-     R_DrawTLColumn8_LinearUV_LinearZ,
-     R_DrawTranslatedColumn8_LinearUV_LinearZ,
-     R_DrawFuzzColumn8_LinearUV_LinearZ,},
-    {R_DrawColumn8_RoundedUV_LinearZ,
-     R_DrawTLColumn8_RoundedUV_LinearZ,
-     R_DrawTranslatedColumn8_RoundedUV_LinearZ,
-     R_DrawFuzzColumn8_RoundedUV_LinearZ,},
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn16_PointUV,
+       R_DrawTLColumn16_PointUV,
+       R_DrawTranslatedColumn16_PointUV,
+       R_DrawFuzzColumn16_PointUV,},
+      {R_DrawColumn16_LinearUV,
+       R_DrawTLColumn16_LinearUV,
+       R_DrawTranslatedColumn16_LinearUV,
+       R_DrawFuzzColumn16_LinearUV,},
+      {R_DrawColumn16_RoundedUV,
+       R_DrawTLColumn16_RoundedUV,
+       R_DrawTranslatedColumn16_RoundedUV,
+       R_DrawFuzzColumn16_RoundedUV,},
+    },
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn16_PointUV_PointZ,
+       R_DrawTLColumn16_PointUV_PointZ,
+       R_DrawTranslatedColumn16_PointUV_PointZ,
+       R_DrawFuzzColumn16_PointUV_PointZ,},
+      {R_DrawColumn16_LinearUV_PointZ,
+       R_DrawTLColumn16_LinearUV_PointZ,
+       R_DrawTranslatedColumn16_LinearUV_PointZ,
+       R_DrawFuzzColumn16_LinearUV_PointZ,},
+      {R_DrawColumn16_RoundedUV_PointZ,
+       R_DrawTLColumn16_RoundedUV_PointZ,
+       R_DrawTranslatedColumn16_RoundedUV_PointZ,
+       R_DrawFuzzColumn16_RoundedUV_PointZ,},
+    },
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn16_PointUV_LinearZ,
+       R_DrawTLColumn16_PointUV_LinearZ,
+       R_DrawTranslatedColumn16_PointUV_LinearZ,
+       R_DrawFuzzColumn16_PointUV_LinearZ,},
+      {R_DrawColumn16_LinearUV_LinearZ,
+       R_DrawTLColumn16_LinearUV_LinearZ,
+       R_DrawTranslatedColumn16_LinearUV_LinearZ,
+       R_DrawFuzzColumn16_LinearUV_LinearZ,},
+      {R_DrawColumn16_RoundedUV_LinearZ,
+       R_DrawTLColumn16_RoundedUV_LinearZ,
+       R_DrawTranslatedColumn16_RoundedUV_LinearZ,
+       R_DrawFuzzColumn16_RoundedUV_LinearZ,},
+    },
+  },
+  {
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn32_PointUV,
+       R_DrawTLColumn32_PointUV,
+       R_DrawTranslatedColumn32_PointUV,
+       R_DrawFuzzColumn32_PointUV,},
+      {R_DrawColumn32_LinearUV,
+       R_DrawTLColumn32_LinearUV,
+       R_DrawTranslatedColumn32_LinearUV,
+       R_DrawFuzzColumn32_LinearUV,},
+      {R_DrawColumn32_RoundedUV,
+       R_DrawTLColumn32_RoundedUV,
+       R_DrawTranslatedColumn32_RoundedUV,
+       R_DrawFuzzColumn32_RoundedUV,},
+    },
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn32_PointUV_PointZ,
+       R_DrawTLColumn32_PointUV_PointZ,
+       R_DrawTranslatedColumn32_PointUV_PointZ,
+       R_DrawFuzzColumn32_PointUV_PointZ,},
+      {R_DrawColumn32_LinearUV_PointZ,
+       R_DrawTLColumn32_LinearUV_PointZ,
+       R_DrawTranslatedColumn32_LinearUV_PointZ,
+       R_DrawFuzzColumn32_LinearUV_PointZ,},
+      {R_DrawColumn32_RoundedUV_PointZ,
+       R_DrawTLColumn32_RoundedUV_PointZ,
+       R_DrawTranslatedColumn32_RoundedUV_PointZ,
+       R_DrawFuzzColumn32_RoundedUV_PointZ,},
+    },
+    {
+      {NULL, NULL, NULL, NULL,},
+      {R_DrawColumn32_PointUV_LinearZ,
+       R_DrawTLColumn32_PointUV_LinearZ,
+       R_DrawTranslatedColumn32_PointUV_LinearZ,
+       R_DrawFuzzColumn32_PointUV_LinearZ,},
+      {R_DrawColumn32_LinearUV_LinearZ,
+       R_DrawTLColumn32_LinearUV_LinearZ,
+       R_DrawTranslatedColumn32_LinearUV_LinearZ,
+       R_DrawFuzzColumn32_LinearUV_LinearZ,},
+      {R_DrawColumn32_RoundedUV_LinearZ,
+       R_DrawTLColumn32_RoundedUV_LinearZ,
+       R_DrawTranslatedColumn32_RoundedUV_LinearZ,
+       R_DrawFuzzColumn32_RoundedUV_LinearZ,},
+    },
   },
 };
 
 R_DrawColumn_f R_GetDrawColumnFunc(enum column_pipeline_e type,
                                    enum draw_filter_type_e filter,
                                    enum draw_filter_type_e filterz) {
-  R_DrawColumn_f result = drawcolumnfuncs[filterz][filter][type];
+  R_DrawColumn_f result = drawcolumnfuncs[V_GetMode()][filterz][filter][type];
   if (result == NULL)
     I_Error("R_GetDrawColumnFunc: undefined function (%d, %d, %d)",
             type, filter, filterz);
@@ -698,59 +745,235 @@ void R_InitTranslationTables (void)
 //
 
 #define R_DRAWSPAN_FUNCNAME R_DrawSpan8_PointUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 8
 #define R_DRAWSPAN_PIPELINE (RDC_STANDARD)
 #include "r_drawspan.inl"
 
 #define R_DRAWSPAN_FUNCNAME R_DrawSpan8_PointUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 8
 #define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_DITHERZ)
 #include "r_drawspan.inl"
 
 #define R_DRAWSPAN_FUNCNAME R_DrawSpan8_LinearUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 8
 #define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_BILINEAR)
 #include "r_drawspan.inl"
 
 #define R_DRAWSPAN_FUNCNAME R_DrawSpan8_LinearUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 8
 #define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_BILINEAR | RDC_DITHERZ)
 #include "r_drawspan.inl"
 
 #define R_DRAWSPAN_FUNCNAME R_DrawSpan8_RoundedUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 8
 #define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_ROUNDED)
 #include "r_drawspan.inl"
 
 #define R_DRAWSPAN_FUNCNAME R_DrawSpan8_RoundedUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 8
 #define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_ROUNDED | RDC_DITHERZ)
 #include "r_drawspan.inl"
 
-static R_DrawSpan_f drawspanfuncs[RDRAW_FILTER_MAXFILTERS][RDRAW_FILTER_MAXFILTERS] = {
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan15_PointUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 15
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan15_PointUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 15
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan15_LinearUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 15
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_BILINEAR)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan15_LinearUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 15
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_BILINEAR | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan15_RoundedUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 15
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_ROUNDED)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan15_RoundedUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 15
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_ROUNDED | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan16_PointUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 16
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan16_PointUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 16
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan16_LinearUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 16
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_BILINEAR)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan16_LinearUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 16
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_BILINEAR | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan16_RoundedUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 16
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_ROUNDED)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan16_RoundedUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 16
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_ROUNDED | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan32_PointUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 32
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan32_PointUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 32
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan32_LinearUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 32
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_BILINEAR)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan32_LinearUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 32
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_BILINEAR | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan32_RoundedUV_PointZ
+#define R_DRAWSPAN_PIPELINE_BITS 32
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_ROUNDED)
+#include "r_drawspan.inl"
+
+#define R_DRAWSPAN_FUNCNAME R_DrawSpan32_RoundedUV_LinearZ
+#define R_DRAWSPAN_PIPELINE_BITS 32
+#define R_DRAWSPAN_PIPELINE (RDC_STANDARD | RDC_ROUNDED | RDC_DITHERZ)
+#include "r_drawspan.inl"
+
+static R_DrawSpan_f drawspanfuncs[VID_MODEMAX][RDRAW_FILTER_MAXFILTERS][RDRAW_FILTER_MAXFILTERS] = {
   {
-    NULL,
-    NULL,
-    NULL,
-    NULL,
+    {
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+    },
+    {
+      NULL,
+      R_DrawSpan8_PointUV_PointZ,
+      R_DrawSpan8_LinearUV_PointZ,
+      R_DrawSpan8_RoundedUV_PointZ,
+    },
+    {
+      NULL,
+      R_DrawSpan8_PointUV_LinearZ,
+      R_DrawSpan8_LinearUV_LinearZ,
+      R_DrawSpan8_RoundedUV_LinearZ,
+    },
+    {
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+    },
   },
   {
-    NULL,
-    R_DrawSpan8_PointUV_PointZ,
-    R_DrawSpan8_LinearUV_PointZ,
-    R_DrawSpan8_RoundedUV_PointZ,
+    {
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+    },
+    {
+      NULL,
+      R_DrawSpan15_PointUV_PointZ,
+      R_DrawSpan15_LinearUV_PointZ,
+      R_DrawSpan15_RoundedUV_PointZ,
+    },
+    {
+      NULL,
+      R_DrawSpan15_PointUV_LinearZ,
+      R_DrawSpan15_LinearUV_LinearZ,
+      R_DrawSpan15_RoundedUV_LinearZ,
+    },
+    {
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+    },
   },
   {
-    NULL,
-    R_DrawSpan8_PointUV_LinearZ,
-    R_DrawSpan8_LinearUV_LinearZ,
-    R_DrawSpan8_RoundedUV_LinearZ,
+    {
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+    },
+    {
+      NULL,
+      R_DrawSpan16_PointUV_PointZ,
+      R_DrawSpan16_LinearUV_PointZ,
+      R_DrawSpan16_RoundedUV_PointZ,
+    },
+    {
+      NULL,
+      R_DrawSpan16_PointUV_LinearZ,
+      R_DrawSpan16_LinearUV_LinearZ,
+      R_DrawSpan16_RoundedUV_LinearZ,
+    },
+    {
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+    },
   },
   {
-    NULL,
-    NULL,
-    NULL,
-    NULL,
+    {
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+    },
+    {
+      NULL,
+      R_DrawSpan32_PointUV_PointZ,
+      R_DrawSpan32_LinearUV_PointZ,
+      R_DrawSpan32_RoundedUV_PointZ,
+    },
+    {
+      NULL,
+      R_DrawSpan32_PointUV_LinearZ,
+      R_DrawSpan32_LinearUV_LinearZ,
+      R_DrawSpan32_RoundedUV_LinearZ,
+    },
+    {
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+    },
   },
 };
 
 R_DrawSpan_f R_GetDrawSpanFunc(enum draw_filter_type_e filter,
                                enum draw_filter_type_e filterz) {
-  R_DrawSpan_f result = drawspanfuncs[filterz][filter];
+  R_DrawSpan_f result = drawspanfuncs[V_GetMode()][filterz][filter];
   if (result == NULL)
     I_Error("R_GetDrawSpanFunc: undefined function (%d, %d)",
             filter, filterz);
@@ -782,11 +1005,23 @@ void R_InitBuffer(int width, int height)
 
   viewwindowy = width==SCREENWIDTH ? 0 : (SCREENHEIGHT-(ST_SCALED_HEIGHT-1)-height)>>1;
 
-  drawvars.topleft = screens[0].data + viewwindowy*screens[0].pitch + viewwindowx;
-  drawvars.pitch = screens[0].pitch;
+  drawvars.byte_topleft = screens[0].data + viewwindowy*screens[0].byte_pitch + viewwindowx;
+  drawvars.short_topleft = (unsigned short *)(screens[0].data) + viewwindowy*screens[0].short_pitch + viewwindowx;
+  drawvars.int_topleft = (unsigned int *)(screens[0].data) + viewwindowy*screens[0].int_pitch + viewwindowx;
+  drawvars.byte_pitch = screens[0].byte_pitch;
+  drawvars.short_pitch = screens[0].short_pitch;
+  drawvars.int_pitch = screens[0].int_pitch;
 
-  for (i=0; i<FUZZTABLE; i++)
-    fuzzoffset[i] = fuzzoffset_org[i]*screens[0].pitch;
+  if (V_GetMode() == VID_MODE8) {
+    for (i=0; i<FUZZTABLE; i++)
+      fuzzoffset[i] = fuzzoffset_org[i]*screens[0].byte_pitch;
+  } else if ((V_GetMode() == VID_MODE15) || (V_GetMode() == VID_MODE16)) {
+    for (i=0; i<FUZZTABLE; i++)
+      fuzzoffset[i] = fuzzoffset_org[i]*screens[0].short_pitch;
+  } else if (V_GetMode() == VID_MODE32) {
+    for (i=0; i<FUZZTABLE; i++)
+      fuzzoffset[i] = fuzzoffset_org[i]*screens[0].int_pitch;
+  }
 }
 
 //
@@ -835,7 +1070,9 @@ void R_FillBackScreen (void)
 void R_VideoErase(int x, int y, int count)
 {
   if (V_GetMode() != VID_MODEGL)
-    memcpy(screens[0].data+y*screens[0].pitch+x, screens[1].data+y*screens[1].pitch+x, count);   // LFB copy.
+    memcpy(screens[0].data+y*screens[0].byte_pitch+x*V_GetPixelDepth(),
+           screens[1].data+y*screens[1].byte_pitch+x*V_GetPixelDepth(),
+           count*V_GetPixelDepth());   // LFB copy.
 }
 
 //
