@@ -106,8 +106,12 @@ typedef struct {
 } draw_span_vars_t;
 
 typedef struct {
-  byte  *topleft;
-  int   pitch;
+  byte           *byte_topleft;
+  unsigned short *short_topleft;
+  unsigned int   *int_topleft;
+  int   byte_pitch;
+  int   short_pitch;
+  int   int_pitch;
 
   enum draw_filter_type_e filterwall;
   enum draw_filter_type_e filterfloor;
