@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -42,9 +44,9 @@
 #define PL_SKYFLAT (0x80000000)
 
 /* Visplane related. */
-extern  short *lastopening;
+extern int *lastopening; // dropoff overflow
 
-extern short floorclip[], ceilingclip[];
+extern int floorclip[], ceilingclip[]; // dropoff overflow
 extern fixed_t yslope[], distscale[];
 
 void R_InitPlanes(void);
@@ -52,7 +54,7 @@ void R_ClearPlanes(void);
 void R_DrawPlanes (void);
 
 visplane_t *R_FindPlane(
-                        fixed_t height, 
+                        fixed_t height,
                         int picnum,
                         int lightlevel,
                         fixed_t xoffs,  /* killough 2/28/98: add x-y offsets */
