@@ -298,6 +298,8 @@ static void R_RenderSegLoop (void)
               ceilingplane->top[rw_x] = top;
               ceilingplane->bottom[rw_x] = bottom;
             }
+          // SoM: this should be set here
+          ceilingclip[rw_x] = bottom;
         }
 
 //      yh = bottomfrac>>HEIGHTBITS;
@@ -316,6 +318,8 @@ static void R_RenderSegLoop (void)
               floorplane->top[rw_x] = top;
               floorplane->bottom[rw_x] = bottom;
             }
+          // SoM: This should be set here to prevent overdraw
+          floorclip[rw_x] = top;
         }
 
       // texturecolumn and lighting are independent of wall tiers
