@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -50,18 +52,13 @@
 // needed for texture pegging
 extern fixed_t *textureheight;
 
-extern lighttable_t **colormaps;          // killough 3/20/98, 4/4/98
-extern lighttable_t *fullcolormap;        // killough 3/20/98
-
-extern int viewwidth;
 extern int scaledviewwidth;
-extern int viewheight;
 
-extern int firstflat;
+extern int firstflat, numflats;
 
 // for global animation
-extern int *flattranslation;    
-extern int *texturetranslation; 
+extern int *flattranslation;
+extern int *texturetranslation;
 
 // Sprite....
 extern int firstspritelump;
@@ -95,9 +92,6 @@ extern line_t           *lines;
 extern int              numsides;
 extern side_t           *sides;
 
-        // sf: for scripting
-extern int              numthings;
-extern mobj_t           **spawnedthings;
 
 //
 // POV data.
@@ -116,9 +110,6 @@ extern angle_t          rw_normalangle;
 // angle to line origin
 extern int              rw_angle1;
 
-// Segs count?
-extern int              sscount;
-
 extern visplane_t       *floorplane;
 extern visplane_t       *ceilingplane;
 
