@@ -296,8 +296,14 @@ static vissprite_t *R_NewVisSprite(void)
 {
   if (num_vissprite >= num_vissprite_alloc)             // killough
     {
+      size_t num_vissprite_alloc_prev = num_vissprite_alloc;
+
       num_vissprite_alloc = num_vissprite_alloc ? num_vissprite_alloc*2 : 128;
       vissprites = realloc(vissprites,num_vissprite_alloc*sizeof(*vissprites));
+      
+      //e6y: set all fields to zero
+      memset(vissprites + num_vissprite_alloc_prev, 0,
+        (num_vissprite_alloc - num_vissprite_alloc_prev)*sizeof(*vissprites));
     }
  return vissprites + num_vissprite++;
 }
@@ -429,11 +435,6 @@ static void R_DrawVisSprite(vissprite_t *vis, int x1, int x2)
       texturecolumn = frac>>FRACBITS;
       dcvars.texu = frac;
 
-#ifdef RANGECHECK
-      if (texturecolumn < 0 || texturecolumn >= patch->width)
-        I_Error ("R_DrawSpriteRange: Bad texturecolumn");
-#endif
-
       R_DrawMaskedColumn(
         patch,
         colfunc,
@@ -521,13 +522,12 @@ static void R_ProjectSprite (mobj_t* thing, int lightlevel)
              thing->frame);
 #endif
 
-  sprframe = &sprdef->spriteframes[thing->frame & FF_FRAMEMASK];
-
-  // If sprframe is NULL the sprite is not in the IWAD.
-  if (!sprframe)
+  if (!sprdef->spriteframes)
     I_Error ("R_ProjectSprite: Missing spriteframes %i : %i", thing->sprite,
              thing->frame);
 
+  sprframe = &sprdef->spriteframes[thing->frame & FF_FRAMEMASK];
+
   if (sprframe->rotate)
     {
       // choose a different rotation based on player view
@@ -569,8 +569,10 @@ static void R_ProjectSprite (mobj_t* thing, int lightlevel)
     return;
 
   // killough 4/9/98: clip things which are out of view due to height
-  if (fz  > viewz + FixedDiv(centeryfrac, xscale) ||
-      gzt < viewz - FixedDiv(centeryfrac-viewheight, xscale))
+  // e6y: fix of hanging decoration disappearing in Batman Doom MAP02
+  // centeryfrac -> viewheightfrac
+  if (fz  > viewz + FixedDiv(viewheightfrac, xscale) ||
+      gzt < viewz - FixedDiv(viewheightfrac-viewheight, xscale))
     return;
 
     // killough 3/27/98: exclude things totally separated
@@ -773,7 +775,9 @@ static void R_DrawPSprite (pspdef_t *psp, int lightlevel)
   else if (psp->state->frame & FF_FULLBRIGHT)
     vis->colormap = fullcolormap;            // full bright // killough 3/20/98
   else
-    vis->colormap = R_ColourMap(lightlevel,pspritescale);  // local light
+    // add a fudge factor to better match the original game
+    vis->colormap = R_ColourMap(lightlevel,
+        FixedMul(pspritescale, 0x2b000));  // local light
 
   // proff 11/99: don't use software stuff in OpenGL
   if (V_GetMode() != VID_MODEGL)
