@@ -587,6 +587,22 @@ int S_AdjustSoundParams(mobj_t *listener, mobj_t *source,
   if (!snd_card || nosfxparm)
     return 0;
 
+  // e6y
+  // Fix crash when the program wants to S_AdjustSoundParams() for player
+  // which is not displayplayer and displayplayer was not spawned at the moment.
+  // It happens in multiplayer demos only.
+  //
+  // Stack trace is:
+  // P_SetupLevel() \ P_LoadThings() \ P_SpawnMapThing() \ P_SpawnPlayer(players[0]) \
+  // P_SetupPsprites() \ P_BringUpWeapon() \ S_StartSound(players[0]->mo, sfx_sawup) \
+  // S_StartSoundAtVolume() \ S_AdjustSoundParams(players[displayplayer]->mo, ...);
+  // players[displayplayer]->mo is NULL
+  //
+  // There is no more crash on e1cmnet3.lmp between e1m2 and e1m3
+  // http://competn.doom2.net/pub/compet-n/doom/coop/movies/e1cmnet3.zip
+  if (!listener)
+    return 0;
+
   // calculate the distance to sound origin
   //  and clip it if necessary
   adx = D_abs(listener->x - source->x);
