@@ -109,7 +109,7 @@ void V_InitColorTranslation(void)
 //
 // No return.
 //
-void V_CopyRect8(int srcx, int srcy, int srcscrn, int width,
+static void FUNC_V_CopyRect(int srcx, int srcy, int srcscrn, int width,
                 int height, int destx, int desty, int destscrn,
                 enum patch_translation_e flags)
 {
@@ -137,14 +137,14 @@ void V_CopyRect8(int srcx, int srcy, int srcscrn, int width,
     I_Error ("V_CopyRect: Bad arguments");
 #endif
 
-  src = screens[srcscrn].data+screens[srcscrn].pitch*srcy+srcx;
-  dest = screens[destscrn].data+screens[destscrn].pitch*desty+destx;
+  src = screens[srcscrn].data+screens[srcscrn].byte_pitch*srcy+srcx*V_GetPixelDepth();
+  dest = screens[destscrn].data+screens[destscrn].byte_pitch*desty+destx*V_GetPixelDepth();
 
   for ( ; height>0 ; height--)
     {
-      memcpy (dest, src, width);
-      src += screens[srcscrn].pitch;
-      dest += screens[destscrn].pitch;
+      memcpy (dest, src, width*V_GetPixelDepth());
+      src += screens[srcscrn].byte_pitch;
+      dest += screens[destscrn].byte_pitch;
     }
 }
 
@@ -154,11 +154,10 @@ void V_CopyRect8(int srcx, int srcy, int srcscrn, int width,
  * cphipps - used to have M_DrawBackground, but that was used the framebuffer
  * directly, so this is my code from the equivalent function in f_finale.c
  */
-void V_DrawBackground8(const char* flatname, int scrn)
+static void FUNC_V_DrawBackground(const char* flatname, int scrn)
 {
   /* erase the entire screen to a tiled background */
   const byte *src;
-  byte       *dest;
   int         x,y;
   int         width,height;
   int         lump;
@@ -168,12 +167,47 @@ void V_DrawBackground8(const char* flatname, int scrn)
 
   /* V_DrawBlock(0, 0, scrn, 64, 64, src, 0); */
   width = height = 64;
-  dest = screens[scrn].data;
+  if (V_GetMode() == VID_MODE8) {
+    byte *dest = screens[scrn].data;
 
-  while (height--) {
-    memcpy (dest, src, width);
-    src += width;
-    dest += screens[scrn].pitch;
+    while (height--) {
+      memcpy (dest, src, width);
+      src += width;
+      dest += screens[scrn].byte_pitch;
+    }
+  } else if (V_GetMode() == VID_MODE15) {
+    unsigned short *dest = (unsigned short *)screens[scrn].data;
+
+    while (height--) {
+      int i;
+      for (i=0; i<width; i++) {
+        dest[i] = VID_PAL15(src[i], VID_COLORWEIGHTMASK);
+      }
+      src += width;
+      dest += screens[scrn].short_pitch;
+    }
+  } else if (V_GetMode() == VID_MODE16) {
+    unsigned short *dest = (unsigned short *)screens[scrn].data;
+
+    while (height--) {
+      int i;
+      for (i=0; i<width; i++) {
+        dest[i] = VID_PAL16(src[i], VID_COLORWEIGHTMASK);
+      }
+      src += width;
+      dest += screens[scrn].short_pitch;
+    }
+  } else if (V_GetMode() == VID_MODE32) {
+    unsigned int *dest = (unsigned int *)screens[scrn].data;
+
+    while (height--) {
+      int i;
+      for (i=0; i<width; i++) {
+        dest[i] = VID_PAL32(src[i], VID_COLORWEIGHTMASK);
+      }
+      src += width;
+      dest += screens[scrn].int_pitch;
+    }
   }
   /* end V_DrawBlock */
 
@@ -201,12 +235,14 @@ void V_Init (void)
     screens[i].not_on_heap = false;
     screens[i].width = 0;
     screens[i].height = 0;
-    screens[i].pitch = 0;
+    screens[i].byte_pitch = 0;
+    screens[i].short_pitch = 0;
+    screens[i].int_pitch = 0;
   }
 }
 
 //
-// V_DrawMemPatch8
+// V_DrawMemPatch
 //
 // CPhipps - unifying patch drawing routine, handles all cases and combinations
 //  of stretching, flipping and translating
@@ -217,7 +253,7 @@ void V_Init (void)
 // (indeed, laziness of the people who wrote the 'clones' of the original V_DrawPatch
 //  means that their inner loops weren't so well optimised, so merging code may even speed them).
 //
-static void V_DrawMemPatch8(int x, int y, int scrn, const rpatch_t *patch,
+static void V_DrawMemPatch(int x, int y, int scrn, const rpatch_t *patch,
         int cm, enum patch_translation_e flags)
 {
   const byte *trans;
@@ -238,9 +274,9 @@ static void V_DrawMemPatch8(int x, int y, int scrn, const rpatch_t *patch,
   if (!trans)
     flags &= ~VPT_TRANS;
 
-  if (!(flags & VPT_STRETCH)) {
+  if (V_GetMode() == VID_MODE8 && !(flags & VPT_STRETCH)) {
     int             col;
-    byte           *desttop = screens[scrn].data+y*screens[scrn].pitch+x;
+    byte           *desttop = screens[scrn].data+y*screens[scrn].byte_pitch+x*V_GetPixelDepth();
     unsigned int    w = patch->width;
 
     if (y<0 || y+patch->height > ((flags & VPT_STRETCH) ? 200 :  SCREENHEIGHT)) {
@@ -254,7 +290,8 @@ static void V_DrawMemPatch8(int x, int y, int scrn, const rpatch_t *patch,
 
     for (col=0 ; (unsigned int)col<=w ; desttop++, col++, x++) {
       int i;
-      const rcolumn_t *column = &patch->columns[(flags & VPT_FLIP) ? w-col : col];
+      const int colindex = (flags & VPT_FLIP) ? (w - col) : (col);
+      const rcolumn_t *column = R_GetPatchColumn(patch, colindex);
 
       if (x < 0)
         continue;
@@ -264,60 +301,60 @@ static void V_DrawMemPatch8(int x, int y, int scrn, const rpatch_t *patch,
       // step through the posts in a column
       for (i=0; i<column->numPosts; i++) {
         const rpost_t *post = &column->posts[i];
-  // killough 2/21/98: Unrolled and performance-tuned
+        // killough 2/21/98: Unrolled and performance-tuned
 
-  register const byte *source = column->pixels + post->topdelta;
-  register byte *dest = desttop + post->topdelta*screens[scrn].pitch;
-  register int count = post->length;
+        const byte *source = column->pixels + post->topdelta;
+        byte *dest = desttop + post->topdelta*screens[scrn].byte_pitch;
+        int count = post->length;
 
-  if (!(flags & VPT_TRANS)) {
-    if ((count-=4)>=0)
-      do {
-        register byte s0,s1;
-        s0 = source[0];
-        s1 = source[1];
-        dest[0] = s0;
-        dest[screens[scrn].pitch] = s1;
-        dest += screens[scrn].pitch*2;
-        s0 = source[2];
-        s1 = source[3];
-        source += 4;
-        dest[0] = s0;
-        dest[screens[scrn].pitch] = s1;
-        dest += screens[scrn].pitch*2;
-      } while ((count-=4)>=0);
-    if (count+=4)
-      do {
-        *dest = *source++;
-        dest += screens[scrn].pitch;
-      } while (--count);
-  } else {
-    // CPhipps - merged translation code here
-    if ((count-=4)>=0)
-      do {
-        register byte s0,s1;
-        s0 = source[0];
-        s1 = source[1];
-        s0 = trans[s0];
-        s1 = trans[s1];
-        dest[0] = s0;
-        dest[screens[scrn].pitch] = s1;
-        dest += screens[scrn].pitch*2;
-        s0 = source[2];
-        s1 = source[3];
-        s0 = trans[s0];
-        s1 = trans[s1];
-        source += 4;
-        dest[0] = s0;
-        dest[screens[scrn].pitch] = s1;
-        dest += screens[scrn].pitch*2;
-      } while ((count-=4)>=0);
-    if (count+=4)
-      do {
-        *dest = trans[*source++];
-        dest += screens[scrn].pitch;
-      } while (--count);
-  }
+        if (!(flags & VPT_TRANS)) {
+          if ((count-=4)>=0)
+            do {
+              register byte s0,s1;
+              s0 = source[0];
+              s1 = source[1];
+              dest[0] = s0;
+              dest[screens[scrn].byte_pitch] = s1;
+              dest += screens[scrn].byte_pitch*2;
+              s0 = source[2];
+              s1 = source[3];
+              source += 4;
+              dest[0] = s0;
+              dest[screens[scrn].byte_pitch] = s1;
+              dest += screens[scrn].byte_pitch*2;
+            } while ((count-=4)>=0);
+          if (count+=4)
+            do {
+              *dest = *source++;
+              dest += screens[scrn].byte_pitch;
+            } while (--count);
+        } else {
+          // CPhipps - merged translation code here
+          if ((count-=4)>=0)
+            do {
+              register byte s0,s1;
+              s0 = source[0];
+              s1 = source[1];
+              s0 = trans[s0];
+              s1 = trans[s1];
+              dest[0] = s0;
+              dest[screens[scrn].byte_pitch] = s1;
+              dest += screens[scrn].byte_pitch*2;
+              s0 = source[2];
+              s1 = source[3];
+              s0 = trans[s0];
+              s1 = trans[s1];
+              source += 4;
+              dest[0] = s0;
+              dest[screens[scrn].byte_pitch] = s1;
+              dest += screens[scrn].byte_pitch*2;
+            } while ((count-=4)>=0);
+          if (count+=4)
+            do {
+              *dest = trans[*source++];
+              dest += screens[scrn].byte_pitch;
+            } while (--count);
+        }
       }
     }
   }
@@ -338,8 +375,19 @@ static void V_DrawMemPatch8(int x, int y, int scrn, const rpatch_t *patch,
 
     R_SetDefaultDrawColumnVars(&dcvars);
 
-    drawvars.topleft = screens[scrn].data;
-    drawvars.pitch = screens[scrn].pitch;
+    drawvars.byte_topleft = screens[scrn].data;
+    drawvars.short_topleft = (unsigned short *)screens[scrn].data;
+    drawvars.int_topleft = (unsigned int *)screens[scrn].data;
+    drawvars.byte_pitch = screens[scrn].byte_pitch;
+    drawvars.short_pitch = screens[scrn].short_pitch;
+    drawvars.int_pitch = screens[scrn].int_pitch;
+
+    if (!(flags & VPT_STRETCH)) {
+      DX = 1 << 16;
+      DXI = 1 << 16;
+      DY = 1 << 16;
+      DYI = 1 << 16;
+    }
 
     if (flags & VPT_TRANS) {
       colfunc = R_GetDrawColumnFunc(RDC_PIPELINE_TRANSLATED, drawvars.filterpatch, RDRAW_FILTER_NONE);
@@ -355,7 +403,7 @@ static void V_DrawMemPatch8(int x, int y, int scrn, const rpatch_t *patch,
 
     dcvars.texheight = patch->height;
     dcvars.iscale = DYI;
-    dcvars.drawingmasked = max(patch->width, patch->height) > 8;
+    dcvars.drawingmasked = MAX(patch->width, patch->height) > 8;
     dcvars.edgetype = drawvars.patch_edges;
 
     if (drawvars.filterpatch == RDRAW_FILTER_LINEAR) {
@@ -439,13 +487,178 @@ static void V_DrawMemPatch8(int x, int y, int scrn, const rpatch_t *patch,
 // static inline; other compilers have different behaviour.
 // This inline is _only_ for the function below
 
-void V_DrawNumPatch8(int x, int y, int scrn, int lump,
+static void FUNC_V_DrawNumPatch(int x, int y, int scrn, int lump,
          int cm, enum patch_translation_e flags)
 {
-  V_DrawMemPatch8(x, y, scrn, R_CachePatchNum(lump), cm, flags);
+  V_DrawMemPatch(x, y, scrn, R_CachePatchNum(lump), cm, flags);
   R_UnlockPatchNum(lump);
 }
 
+unsigned short *V_Palette15 = NULL;
+unsigned short *V_Palette16 = NULL;
+unsigned int *V_Palette32 = NULL;
+static unsigned short *Palettes15 = NULL;
+static unsigned short *Palettes16 = NULL;
+static unsigned int *Palettes32 = NULL;
+static int currentPaletteIndex = 0;
+
+//
+// V_UpdateTrueColorPalette
+//
+void V_UpdateTrueColorPalette(video_mode_t mode) {
+  int i, w, p;
+  byte r,g,b;
+  int nr,ng,nb;
+  float t;
+  int paletteNum = (V_GetMode() == VID_MODEGL ? 0 : currentPaletteIndex);
+  static int usegammaOnLastPaletteGeneration = -1;
+  
+  int pplump = W_GetNumForName("PLAYPAL");
+  int gtlump = (W_CheckNumForName)("GAMMATBL",ns_prboom);
+  const byte *pal = W_CacheLumpNum(pplump);
+  // opengl doesn't use the gamma
+  const byte *const gtable = 
+    (const byte *)W_CacheLumpNum(gtlump) + 
+    (V_GetMode() == VID_MODEGL ? 0 : 256*(usegamma))
+  ;
+
+  int numPals = W_LumpLength(pplump) / (3*256);
+  const float dontRoundAbove = 220;
+  float roundUpR, roundUpG, roundUpB;
+  
+  if (usegammaOnLastPaletteGeneration != usegamma) {
+    if (Palettes15) free(Palettes15);
+    if (Palettes16) free(Palettes16);
+    if (Palettes32) free(Palettes32);
+    Palettes15 = NULL;
+    Palettes16 = NULL;
+    Palettes32 = NULL;
+    usegammaOnLastPaletteGeneration = usegamma;      
+  }
+  
+  if (mode == VID_MODE32) {
+    if (!Palettes32) {
+      // set int palette
+      Palettes32 = (int*)malloc(numPals*256*sizeof(int)*VID_NUMCOLORWEIGHTS);
+      for (p=0; p<numPals; p++) {
+        for (i=0; i<256; i++) {
+          r = gtable[pal[(256*p+i)*3+0]];
+          g = gtable[pal[(256*p+i)*3+1]];
+          b = gtable[pal[(256*p+i)*3+2]];
+          
+          // ideally, we should always round up, but very bright colors
+          // overflow the blending adds, so they don't get rounded.
+          roundUpR = (r > dontRoundAbove) ? 0 : 0.5f;
+          roundUpG = (g > dontRoundAbove) ? 0 : 0.5f;
+          roundUpB = (b > dontRoundAbove) ? 0 : 0.5f;
+                  
+          for (w=0; w<VID_NUMCOLORWEIGHTS; w++) {
+            t = (float)(w)/(float)(VID_NUMCOLORWEIGHTS-1);
+            nr = (int)(r*t+roundUpR);
+            ng = (int)(g*t+roundUpG);
+            nb = (int)(b*t+roundUpB);
+            Palettes32[((p*256+i)*VID_NUMCOLORWEIGHTS)+w] = (
+              (nr<<16) | (ng<<8) | nb
+            );
+          }
+        }
+      }
+    }
+    V_Palette32 = Palettes32 + paletteNum*256*VID_NUMCOLORWEIGHTS;
+  }
+  else if (mode == VID_MODE16) {
+    if (!Palettes16) {
+      // set short palette
+      Palettes16 = (short*)malloc(numPals*256*sizeof(short)*VID_NUMCOLORWEIGHTS);
+      for (p=0; p<numPals; p++) {
+        for (i=0; i<256; i++) {
+          r = gtable[pal[(256*p+i)*3+0]];
+          g = gtable[pal[(256*p+i)*3+1]];
+          b = gtable[pal[(256*p+i)*3+2]];
+          
+          // ideally, we should always round up, but very bright colors
+          // overflow the blending adds, so they don't get rounded.
+          roundUpR = (r > dontRoundAbove) ? 0 : 0.5f;
+          roundUpG = (g > dontRoundAbove) ? 0 : 0.5f;
+          roundUpB = (b > dontRoundAbove) ? 0 : 0.5f;
+                   
+          for (w=0; w<VID_NUMCOLORWEIGHTS; w++) {
+            t = (float)(w)/(float)(VID_NUMCOLORWEIGHTS-1);
+            nr = (int)((r>>3)*t+roundUpR);
+            ng = (int)((g>>2)*t+roundUpG);
+            nb = (int)((b>>3)*t+roundUpB);
+            Palettes16[((p*256+i)*VID_NUMCOLORWEIGHTS)+w] = (
+              (nr<<11) | (ng<<5) | nb
+            );
+          }
+        }
+      }
+    }
+    V_Palette16 = Palettes16 + paletteNum*256*VID_NUMCOLORWEIGHTS;
+  }
+  else if (mode == VID_MODE15) {
+    if (!Palettes15) {
+      // set short palette
+      Palettes15 = (short*)malloc(numPals*256*sizeof(short)*VID_NUMCOLORWEIGHTS);
+      for (p=0; p<numPals; p++) {
+        for (i=0; i<256; i++) {
+          r = gtable[pal[(256*p+i)*3+0]];
+          g = gtable[pal[(256*p+i)*3+1]];
+          b = gtable[pal[(256*p+i)*3+2]];
+          
+          // ideally, we should always round up, but very bright colors
+          // overflow the blending adds, so they don't get rounded.
+          roundUpR = (r > dontRoundAbove) ? 0 : 0.5f;
+          roundUpG = (g > dontRoundAbove) ? 0 : 0.5f;
+          roundUpB = (b > dontRoundAbove) ? 0 : 0.5f;
+                   
+          for (w=0; w<VID_NUMCOLORWEIGHTS; w++) {
+            t = (float)(w)/(float)(VID_NUMCOLORWEIGHTS-1);
+            nr = (int)((r>>3)*t+roundUpR);
+            ng = (int)((g>>3)*t+roundUpG);
+            nb = (int)((b>>3)*t+roundUpB);
+            Palettes15[((p*256+i)*VID_NUMCOLORWEIGHTS)+w] = (
+              (nr<<10) | (ng<<5) | nb
+            );
+          }
+        }
+      }
+    }
+    V_Palette15 = Palettes15 + paletteNum*256*VID_NUMCOLORWEIGHTS;
+  }       
+   
+  W_UnlockLumpNum(pplump);
+  W_UnlockLumpNum(gtlump);
+}
+
+
+//---------------------------------------------------------------------------
+// V_DestroyTrueColorPalette
+//---------------------------------------------------------------------------
+static void V_DestroyTrueColorPalette(video_mode_t mode) {
+  if (mode == VID_MODE15) {
+    if (Palettes15) free(Palettes15);
+    Palettes15 = NULL;
+    V_Palette15 = NULL;
+  }
+  if (mode == VID_MODE16) {
+    if (Palettes16) free(Palettes16);
+    Palettes16 = NULL;
+    V_Palette16 = NULL;
+  }
+  if (mode == VID_MODE32) {
+    if (Palettes32) free(Palettes32);
+    Palettes32 = NULL;
+    V_Palette32 = NULL;
+  }
+}
+
+void V_DestroyUnusedTrueColorPalettes(void) {
+  if (V_GetMode() != VID_MODE15) V_DestroyTrueColorPalette(VID_MODE15);
+  if (V_GetMode() != VID_MODE16) V_DestroyTrueColorPalette(VID_MODE16);
+  if (V_GetMode() != VID_MODE32) V_DestroyTrueColorPalette(VID_MODE32);  
+}
+
 //
 // V_SetPalette
 //
@@ -454,12 +667,21 @@ void V_DrawNumPatch8(int x, int y, int scrn, int lump,
 
 void V_SetPalette(int pal)
 {
+  currentPaletteIndex = pal;
+
   if (V_GetMode() == VID_MODEGL) {
 #ifdef GL_DOOM
     gld_SetPalette(pal);
 #endif
   } else {
     I_SetPalette(pal);
+    if (V_GetMode() == VID_MODE15 || V_GetMode() == VID_MODE16 || V_GetMode() == VID_MODE32) {
+      // V_SetPalette can be called as part of the gamma setting before
+      // we've loaded any wads, which prevents us from reading the palette - POPE
+      if (W_CheckNumForName("PLAYPAL") >= 0) {
+        V_UpdateTrueColorPalette(V_GetMode());
+      }
+    }
   }
 }
 
@@ -467,42 +689,84 @@ void V_SetPalette(int pal)
 // V_FillRect
 //
 // CPhipps - New function to fill a rectangle with a given colour
-void V_FillRect8(int scrn, int x, int y, int width, int height, byte colour)
+static void V_FillRect8(int scrn, int x, int y, int width, int height, byte colour)
 {
-  byte* dest = screens[scrn].data + x + y*screens[scrn].pitch;
+  byte* dest = screens[scrn].data + x + y*screens[scrn].byte_pitch;
   while (height--) {
     memset(dest, colour, width);
-    dest += screens[scrn].pitch;
+    dest += screens[scrn].byte_pitch;
+  }
+}
+
+static void V_FillRect15(int scrn, int x, int y, int width, int height, byte colour)
+{
+  unsigned short* dest = (unsigned short *)screens[scrn].data + x + y*screens[scrn].short_pitch;
+  int w;
+  short c = VID_PAL15(colour, VID_COLORWEIGHTMASK);
+  while (height--) {
+    for (w=0; w<width; w++) {
+      dest[w] = c;
+    }
+    dest += screens[scrn].short_pitch;
+  }
+}
+
+static void V_FillRect16(int scrn, int x, int y, int width, int height, byte colour)
+{
+  unsigned short* dest = (unsigned short *)screens[scrn].data + x + y*screens[scrn].short_pitch;
+  int w;
+  short c = VID_PAL16(colour, VID_COLORWEIGHTMASK);
+  while (height--) {
+    for (w=0; w<width; w++) {
+      dest[w] = c;
+    }
+    dest += screens[scrn].short_pitch;
+  }
+}
+
+static void V_FillRect32(int scrn, int x, int y, int width, int height, byte colour)
+{
+  unsigned int* dest = (unsigned int *)screens[scrn].data + x + y*screens[scrn].int_pitch;
+  int w;
+  int c = VID_PAL32(colour, VID_COLORWEIGHTMASK);
+  while (height--) {
+    for (w=0; w<width; w++) {
+      dest[w] = c;
+    }
+    dest += screens[scrn].int_pitch;
   }
 }
 
 static void WRAP_V_DrawLine(fline_t* fl, int color);
 static void V_PlotPixel8(int scrn, int x, int y, byte color);
+static void V_PlotPixel15(int scrn, int x, int y, byte color);
+static void V_PlotPixel16(int scrn, int x, int y, byte color);
+static void V_PlotPixel32(int scrn, int x, int y, byte color);
 
 #ifdef GL_DOOM
-void WRAP_gld_FillRect(int scrn, int x, int y, int width, int height, byte colour)
+static void WRAP_gld_FillRect(int scrn, int x, int y, int width, int height, byte colour)
 {
   gld_FillBlock(x,y,width,height,colour);
 }
-void WRAP_gld_CopyRect(int srcx, int srcy, int srcscrn, int width, int height, int destx, int desty, int destscrn, enum patch_translation_e flags)
+static void WRAP_gld_CopyRect(int srcx, int srcy, int srcscrn, int width, int height, int destx, int desty, int destscrn, enum patch_translation_e flags)
 {
 }
-void WRAP_gld_DrawBackground(const char *flatname, int n)
+static void WRAP_gld_DrawBackground(const char *flatname, int n)
 {
   gld_DrawBackground(flatname);
 }
-void WRAP_gld_DrawNumPatch(int x, int y, int scrn, int lump, int cm, enum patch_translation_e flags)
+static void WRAP_gld_DrawNumPatch(int x, int y, int scrn, int lump, int cm, enum patch_translation_e flags)
 {
   gld_DrawNumPatch(x,y,lump,cm,flags);
 }
-void WRAP_gld_DrawBlock(int x, int y, int scrn, int width, int height, const byte *src, enum patch_translation_e flags)
+static void WRAP_gld_DrawBlock(int x, int y, int scrn, int width, int height, const byte *src, enum patch_translation_e flags)
 {
 }
-void V_PlotPixelGL(int scrn, int x, int y, byte color) {
+static void V_PlotPixelGL(int scrn, int x, int y, byte color) {
   gld_DrawLine(x-1, y, x+1, y, color);
   gld_DrawLine(x, y-1, x, y+1, color);
 }
-void WRAP_gld_DrawLine(fline_t* fl, int color)
+static void WRAP_gld_DrawLine(fline_t* fl, int color)
 {
   gld_DrawLine(fl->a.x, fl->a.y, fl->b.x, fl->b.y, color);
 }
@@ -516,7 +780,7 @@ static void NULL_DrawBlock(int x, int y, int scrn, int width, int height, const
 static void NULL_PlotPixel(int scrn, int x, int y, byte color) {}
 static void NULL_DrawLine(fline_t* fl, int color) {}
 
-video_mode_t default_videomode;
+const char *default_videomode;
 static video_mode_t current_videomode = VID_MODE8;
 
 V_CopyRect_f V_CopyRect = NULL_CopyRect;
@@ -536,17 +800,45 @@ void V_InitMode(video_mode_t mode) {
 #endif
   switch (mode) {
     case VID_MODE8:
-    //case VID_MODE16:
-    //case VID_MODE32:
       lprintf(LO_INFO, "V_InitMode: using 8 bit video mode\n");
-      V_CopyRect = V_CopyRect8;
+      V_CopyRect = FUNC_V_CopyRect;
       V_FillRect = V_FillRect8;
-      V_DrawNumPatch = V_DrawNumPatch8;
-      V_DrawBackground = V_DrawBackground8;
+      V_DrawNumPatch = FUNC_V_DrawNumPatch;
+      V_DrawBackground = FUNC_V_DrawBackground;
       V_PlotPixel = V_PlotPixel8;
       V_DrawLine = WRAP_V_DrawLine;
       current_videomode = VID_MODE8;
       break;
+    case VID_MODE15:
+      lprintf(LO_INFO, "V_InitMode: using 15 bit video mode\n");
+      V_CopyRect = FUNC_V_CopyRect;
+      V_FillRect = V_FillRect15;
+      V_DrawNumPatch = FUNC_V_DrawNumPatch;
+      V_DrawBackground = FUNC_V_DrawBackground;
+      V_PlotPixel = V_PlotPixel15;
+      V_DrawLine = WRAP_V_DrawLine;
+      current_videomode = VID_MODE15;
+      break;
+    case VID_MODE16:
+      lprintf(LO_INFO, "V_InitMode: using 16 bit video mode\n");
+      V_CopyRect = FUNC_V_CopyRect;
+      V_FillRect = V_FillRect16;
+      V_DrawNumPatch = FUNC_V_DrawNumPatch;
+      V_DrawBackground = FUNC_V_DrawBackground;
+      V_PlotPixel = V_PlotPixel16;
+      V_DrawLine = WRAP_V_DrawLine;
+      current_videomode = VID_MODE16;
+      break;
+    case VID_MODE32:
+      lprintf(LO_INFO, "V_InitMode: using 32 bit video mode\n");
+      V_CopyRect = FUNC_V_CopyRect;
+      V_FillRect = V_FillRect32;
+      V_DrawNumPatch = FUNC_V_DrawNumPatch;
+      V_DrawBackground = FUNC_V_DrawBackground;
+      V_PlotPixel = V_PlotPixel32;
+      V_DrawLine = WRAP_V_DrawLine;
+      current_videomode = VID_MODE32;
+      break;
 #ifdef GL_DOOM
     case VID_MODEGL:
       lprintf(LO_INFO, "V_InitMode: using OpenGL video mode\n");
@@ -573,11 +865,12 @@ video_mode_t V_GetMode(void) {
 //
 // V_GetModePixelDepth
 //
-static int V_GetModePixelDepth(video_mode_t mode) {
+int V_GetModePixelDepth(video_mode_t mode) {
   switch (mode) {
     case VID_MODE8: return 1;
-    //case VID_MODE16: return 2;
-    //case VID_MODE32: return 4;
+    case VID_MODE15: return 2;
+    case VID_MODE16: return 2;
+    case VID_MODE32: return 4;
     default: return 0;
   }
 }
@@ -586,7 +879,13 @@ static int V_GetModePixelDepth(video_mode_t mode) {
 // V_GetNumPixelBits
 //
 int V_GetNumPixelBits(void) {
-  return V_GetModePixelDepth(current_videomode) * 8;
+  switch (current_videomode) {
+    case VID_MODE8: return 8;
+    case VID_MODE15: return 15;
+    case VID_MODE16: return 16;
+    case VID_MODE32: return 32;
+    default: return 0;
+  }
 }
 
 //
@@ -601,8 +900,8 @@ int V_GetPixelDepth(void) {
 //
 void V_AllocScreen(screeninfo_t *scrn) {
   if (!scrn->not_on_heap)
-    if ((scrn->pitch * scrn->height) > 0)
-      scrn->data = malloc(scrn->pitch*scrn->height);
+    if ((scrn->byte_pitch * scrn->height) > 0)
+      scrn->data = malloc(scrn->byte_pitch*scrn->height);
 }
 
 //
@@ -635,8 +934,20 @@ void V_FreeScreens(void) {
     V_FreeScreen(&screens[i]);
 }
 
-void V_PlotPixel8(int scrn, int x, int y, byte color) {
-  screens[scrn].data[x+screens[scrn].pitch*y] = color;
+static void V_PlotPixel8(int scrn, int x, int y, byte color) {
+  screens[scrn].data[x+screens[scrn].byte_pitch*y] = color;
+}
+
+static void V_PlotPixel15(int scrn, int x, int y, byte color) {
+  ((unsigned short *)screens[scrn].data)[x+screens[scrn].short_pitch*y] = VID_PAL15(color, VID_COLORWEIGHTMASK);
+}
+
+static void V_PlotPixel16(int scrn, int x, int y, byte color) {
+  ((unsigned short *)screens[scrn].data)[x+screens[scrn].short_pitch*y] = VID_PAL16(color, VID_COLORWEIGHTMASK);
+}
+
+static void V_PlotPixel32(int scrn, int x, int y, byte color) {
+  ((unsigned int *)screens[scrn].data)[x+screens[scrn].int_pitch*y] = VID_PAL32(color, VID_COLORWEIGHTMASK);
 }
 
 //
